@@ -51,7 +51,7 @@ impl ClientBuilder {
                 proxy: self.proxy,
                 ratelimiter: self.ratelimiter,
                 timeout: self.timeout,
-                token_invalid: AtomicBool::new(false),
+                token_invalid: Arc::new(AtomicBool::new(false)),
                 token: self.token,
                 application_id: self.application_id,
                 default_allowed_mentions: self.default_allowed_mentions,
@@ -90,14 +90,14 @@ impl ClientBuilder {
     ///
     /// # fn main() -> Result<(), Box<dyn std::error::Error>> {
     /// let client = Client::builder()
-    ///     .proxy("twilight_http_proxy.internal", true)
+    ///     .proxy("twilight_http_proxy.internal".to_owned(), true)
     ///     .build();
     /// # Ok(()) }
     /// ```
     ///
     /// [twilight's HTTP proxy server]: https://github.com/twilight-rs/http-proxy
-    pub fn proxy(mut self, proxy_url: impl Into<String>, use_http: bool) -> Self {
-        self.proxy.replace(proxy_url.into().into_boxed_str());
+    pub fn proxy(mut self, proxy_url: String, use_http: bool) -> Self {
+        self.proxy.replace(proxy_url.into_boxed_str());
         self.use_http = use_http;
 
         self
@@ -110,8 +110,9 @@ impl ClientBuilder {
     ///
     /// If this method is not called at all then a default ratelimiter will be
     /// created by [`ClientBuilder::build`].
-    pub fn ratelimiter(mut self, ratelimiter: impl Into<Option<Ratelimiter>>) -> Self {
-        self.ratelimiter = ratelimiter.into();
+    #[allow(clippy::missing_const_for_fn)]
+    pub fn ratelimiter(mut self, ratelimiter: Option<Ratelimiter>) -> Self {
+        self.ratelimiter = ratelimiter;
 
         self
     }
@@ -133,9 +134,7 @@ impl ClientBuilder {
     }
 
     /// Set the token to use for HTTP requests.
-    pub fn token(mut self, token: impl Into<String>) -> Self {
-        let mut token = token.into();
-
+    pub fn token(mut self, mut token: String) -> Self {
         let is_bot = token.starts_with("Bot ");
         let is_bearer = token.starts_with("Bearer ");
 
