@@ -1,9 +1,8 @@
 #[cfg(not(feature = "simd-json"))]
-pub use serde_json::to_vec;
+pub use serde_json::{to_vec, Deserializer as JsonDeserializer, Error as JsonError};
 #[cfg(feature = "simd-json")]
-pub use simd_json::to_vec;
+pub use simd_json::{to_vec, Deserializer as JsonDeserializer, Error as JsonError};
 
-use crate::error::{Error, ErrorType};
 use hyper::body::Bytes;
 use serde::de::DeserializeOwned;
 
@@ -15,7 +14,7 @@ use simd_json::Result as JsonResult;
 pub fn from_bytes<T: DeserializeOwned>(bytes: &Bytes) -> JsonResult<T> {
     #[cfg(not(feature = "simd-json"))]
     {
-        serde_json::from_slice::<T>(&bytes)
+        serde_json::from_slice::<T>(bytes)
     }
 
     #[cfg(feature = "simd-json")]
@@ -24,12 +23,3 @@ pub fn from_bytes<T: DeserializeOwned>(bytes: &Bytes) -> JsonResult<T> {
         simd_json::from_slice(&mut bytes.to_vec())
     }
 }
-
-pub fn parse_bytes<T: DeserializeOwned>(bytes: &Bytes) -> Result<T, Error> {
-    from_bytes(bytes).map_err(|source| Error {
-        kind: ErrorType::Parsing {
-            body: bytes.to_vec(),
-        },
-        source: Some(Box::new(source)),
-    })
-}
