@@ -93,10 +93,10 @@
     warnings
 )]
 #![allow(
-    clippy::module_name_repetitions,
-    clippy::pub_enum_variant_names,
-    clippy::must_use_candidate,
     clippy::missing_errors_doc,
+    clippy::module_name_repetitions,
+    clippy::must_use_candidate,
+    clippy::semicolon_if_nothing_returned,
     clippy::unnecessary_wraps
 )]
 #![cfg_attr(docsrs, feature(doc_cfg))]
@@ -106,6 +106,7 @@ pub mod client;
 pub mod error;
 pub mod ratelimiting;
 pub mod request;
+pub mod response;
 pub mod routing;
 
 mod json;
@@ -113,7 +114,7 @@ mod json;
 /// Discord API version used by this crate.
 pub const API_VERSION: u8 = 8;
 
-pub use crate::{client::Client, error::Error};
+pub use crate::{client::Client, error::Error, response::Response};
 
 #[cfg(not(any(
     feature = "native",
