@@ -1,7 +1,8 @@
 use crate::{
     client::Client,
     error::Error,
-    request::{Form, Pending, Request},
+    request::{Form, Request},
+    response::ResponseFuture,
     routing::Route,
 };
 use serde::Serialize;
@@ -14,28 +15,28 @@ use twilight_model::{
     id::ApplicationId,
 };
 
-#[derive(Default, Serialize)]
-pub(crate) struct CreateFollowupMessageFields {
+#[derive(Serialize)]
+pub(crate) struct CreateFollowupMessageFields<'a> {
     #[serde(skip_serializing_if = "Option::is_none")]
-    avatar_url: Option<String>,
+    avatar_url: Option<&'a str>,
     #[serde(skip_serializing_if = "Option::is_none")]
-    content: Option<String>,
+    content: Option<&'a str>,
     #[serde(skip_serializing_if = "Option::is_none")]
-    embeds: Option<Vec<Embed>>,
+    embeds: Option<&'a [Embed]>,
     #[serde(skip_serializing_if = "Option::is_none")]
-    payload_json: Option<Vec<u8>>,
+    payload_json: Option<&'a [u8]>,
     #[serde(skip_serializing_if = "Option::is_none")]
     tts: Option<bool>,
     #[serde(skip_serializing_if = "Option::is_none")]
-    username: Option<String>,
+    username: Option<&'a str>,
     #[serde(skip_serializing_if = "Option::is_none")]
     flags: Option<MessageFlags>,
-    allowed_mentions: Option<AllowedMentions>,
+    allowed_mentions: Option<&'a AllowedMentions>,
 }
 
 /// Create a followup message to an interaction.
 ///
-/// You must specify at least one of [`content`], [`embeds`], or [`file`].
+/// You must specify at least one of [`content`], [`embeds`], or [`files`].
 ///
 /// # Examples
 ///
@@ -48,51 +49,59 @@ pub(crate) struct CreateFollowupMessageFields {
 /// let client = Client::new(env::var("DISCORD_TOKEN")?);
 /// client.set_application_id(ApplicationId(1));
 ///
-/// let webhook = client
+/// client
 ///     .create_followup_message("webhook token")?
 ///     .content("Pinkie...")
+///     .exec()
 ///     .await?;
 /// # Ok(()) }
 /// ```
 ///
 /// [`content`]: Self::content
 /// [`embeds`]: Self::embeds
-/// [`file`]: Self::file
+/// [`files`]: Self::files
 pub struct CreateFollowupMessage<'a> {
-    pub(crate) fields: CreateFollowupMessageFields,
-    files: Vec<(String, Vec<u8>)>,
-    fut: Option<Pending<'a, Option<Message>>>,
+    pub(crate) fields: CreateFollowupMessageFields<'a>,
+    files: &'a [(&'a str, &'a [u8])],
     http: &'a Client,
-    token: String,
+    token: &'a str,
     application_id: ApplicationId,
 }
 
 impl<'a> CreateFollowupMessage<'a> {
-    pub(crate) fn new(
+    pub(crate) const fn new(
         http: &'a Client,
         application_id: ApplicationId,
-        token: impl Into<String>,
+        token: &'a str,
     ) -> Self {
         Self {
-            fields: CreateFollowupMessageFields::default(),
-            files: Vec::new(),
-            fut: None,
+            fields: CreateFollowupMessageFields {
+                avatar_url: None,
+                content: None,
+                embeds: None,
+                payload_json: None,
+                tts: None,
+                username: None,
+                flags: None,
+                allowed_mentions: None,
+            },
+            files: &[],
             http,
-            token: token.into(),
+            token,
             application_id,
         }
     }
 
     /// Specify the [`AllowedMentions`] for the webhook message.
-    pub fn allowed_mentions(mut self, allowed_mentions: AllowedMentions) -> Self {
-        self.fields.allowed_mentions.replace(allowed_mentions);
+    pub const fn allowed_mentions(mut self, allowed_mentions: &'a AllowedMentions) -> Self {
+        self.fields.allowed_mentions = Some(allowed_mentions);
 
         self
     }
 
     /// The URL of the avatar of the webhook.
-    pub fn avatar_url(mut self, avatar_url: impl Into<String>) -> Self {
-        self.fields.avatar_url.replace(avatar_url.into());
+    pub const fn avatar_url(mut self, avatar_url: &'a str) -> Self {
+        self.fields.avatar_url = Some(avatar_url);
 
         self
     }
@@ -100,23 +109,23 @@ impl<'a> CreateFollowupMessage<'a> {
     /// The content of the webook's message.
     ///
     /// Up to 2000 UTF-16 codepoints.
-    pub fn content(mut self, content: impl Into<String>) -> Self {
-        self.fields.content.replace(content.into());
+    pub const fn content(mut self, content: &'a str) -> Self {
+        self.fields.content = Some(content);
 
         self
     }
 
     /// Set the list of embeds of the webhook's message.
-    pub fn embeds(mut self, embeds: Vec<Embed>) -> Self {
-        self.fields.embeds.replace(embeds);
+    pub const fn embeds(mut self, embeds: &'a [Embed]) -> Self {
+        self.fields.embeds = Some(embeds);
 
         self
     }
 
     /// Set if the followup should be ephemeral.
-    pub fn ephemeral(mut self, ephemeral: bool) -> Self {
+    pub const fn ephemeral(mut self, ephemeral: bool) -> Self {
         if ephemeral {
-            self.fields.flags.replace(MessageFlags::EPHEMERAL);
+            self.fields.flags = Some(MessageFlags::EPHEMERAL);
         } else {
             self.fields.flags = None;
         }
@@ -124,26 +133,9 @@ impl<'a> CreateFollowupMessage<'a> {
         self
     }
 
-    /// Attach a file to the webhook.
-    ///
-    /// This method is repeatable.
-    #[deprecated(since = "0.5.5", note = "will be removed in favor of `files`")]
-    pub fn file(mut self, name: impl Into<String>, file: impl Into<Vec<u8>>) -> Self {
-        self.files.push((name.into(), file.into()));
-
-        self
-    }
-
     /// Attach multiple files to the webhook.
-    pub fn files<N: Into<String>, F: Into<Vec<u8>>>(
-        mut self,
-        attachments: impl IntoIterator<Item = (N, F)>,
-    ) -> Self {
-        self.files.extend(
-            attachments
-                .into_iter()
-                .map(|(name, file)| (name.into(), file.into())),
-        );
+    pub const fn files(mut self, files: &'a [(&'a str, &'a [u8])]) -> Self {
+        self.files = files;
 
         self
     }
@@ -170,9 +162,11 @@ impl<'a> CreateFollowupMessage<'a> {
     ///
     /// let message = client.create_followup_message("token here")?
     ///     .content("some content")
-    ///     .embeds(vec![EmbedBuilder::new().title("title").build()?])
+    ///     .embeds(&[EmbedBuilder::new().title("title").build()?])
+    ///     .exec()
     ///     .await?
-    ///     .unwrap();
+    ///     .model()
+    ///     .await?;
     ///
     /// assert_eq!(message.content, "some content");
     /// # Ok(()) }
@@ -192,9 +186,11 @@ impl<'a> CreateFollowupMessage<'a> {
     ///
     /// let message = client.create_followup_message("token here")?
     ///     .content("some content")
-    ///     .payload_json(r#"{ "content": "other content", "embeds": [ { "title": "title" } ] }"#)
+    ///     .payload_json(br#"{ "content": "other content", "embeds": [ { "title": "title" } ] }"#)
+    ///     .exec()
     ///     .await?
-    ///     .unwrap();
+    ///     .model()
+    ///     .await?;
     ///
     /// assert_eq!(message.content, "other content");
     /// # Ok(()) }
@@ -202,29 +198,31 @@ impl<'a> CreateFollowupMessage<'a> {
     ///
     /// [`payload_json`]: Self::payload_json
     /// [Discord Docs/Create Message]: https://discord.com/developers/docs/resources/channel#create-message-params
-    pub fn payload_json(mut self, payload_json: impl Into<Vec<u8>>) -> Self {
-        self.fields.payload_json.replace(payload_json.into());
+    pub const fn payload_json(mut self, payload_json: &'a [u8]) -> Self {
+        self.fields.payload_json = Some(payload_json);
 
         self
     }
 
     /// Specify true if the message is TTS.
-    pub fn tts(mut self, tts: bool) -> Self {
-        self.fields.tts.replace(tts);
+    pub const fn tts(mut self, tts: bool) -> Self {
+        self.fields.tts = Some(tts);
 
         self
     }
 
     /// Specify the username of the webhook's message.
-    pub fn username(mut self, username: impl Into<String>) -> Self {
-        self.fields.username.replace(username.into());
+    pub const fn username(mut self, username: &'a str) -> Self {
+        self.fields.username = Some(username);
 
         self
     }
 
-    fn start(&mut self) -> Result<(), Error> {
+    // `self` needs to be consumed and the client returned due to parameters
+    // being consumed in request construction.
+    fn request(&self) -> Result<Request<'a>, Error> {
         let mut request = Request::builder(Route::ExecuteWebhook {
-            token: self.token.clone(),
+            token: self.token,
             wait: None,
             webhook_id: self.application_id.0,
         });
@@ -232,12 +230,12 @@ impl<'a> CreateFollowupMessage<'a> {
         if !self.files.is_empty() || self.fields.payload_json.is_some() {
             let mut form = Form::new();
 
-            for (index, (name, file)) in self.files.drain(..).enumerate() {
-                form.file(format!("{}", index).as_bytes(), name.as_bytes(), &file);
+            for (index, (name, file)) in self.files.iter().enumerate() {
+                form.file(index.to_be_bytes().as_ref(), name.as_bytes(), file);
             }
 
             if let Some(payload_json) = &self.fields.payload_json {
-                form.payload_json(&payload_json);
+                form.payload_json(payload_json);
             } else {
                 let body = crate::json::to_vec(&self.fields).map_err(Error::json)?;
                 form.payload_json(&body);
@@ -248,11 +246,16 @@ impl<'a> CreateFollowupMessage<'a> {
             request = request.json(&self.fields)?;
         }
 
-        self.fut
-            .replace(Box::pin(self.http.request(request.build())));
+        Ok(request.build())
+    }
 
-        Ok(())
+    /// Execute the request, returning a future resolving to a [`Response`].
+    ///
+    /// [`Response`]: crate::response::Response
+    pub fn exec(self) -> ResponseFuture<Message> {
+        match self.request() {
+            Ok(request) => self.http.request(request),
+            Err(source) => ResponseFuture::error(source),
+        }
     }
 }
-
-poll_req!(CreateFollowupMessage<'_>, Option<Message>);
