@@ -1,7 +1,7 @@
 use crate::{
     client::Client,
-    error::Error,
-    request::{Pending, Request},
+    request::Request,
+    response::{marker::EmptyBody, ResponseFuture},
     routing::Route,
 };
 use twilight_model::id::{ApplicationId, CommandId, GuildId};
@@ -10,13 +10,12 @@ use twilight_model::id::{ApplicationId, CommandId, GuildId};
 pub struct DeleteGuildCommand<'a> {
     application_id: ApplicationId,
     command_id: CommandId,
-    fut: Option<Pending<'a, ()>>,
     guild_id: GuildId,
     http: &'a Client,
 }
 
 impl<'a> DeleteGuildCommand<'a> {
-    pub(crate) fn new(
+    pub(crate) const fn new(
         http: &'a Client,
         application_id: ApplicationId,
         guild_id: GuildId,
@@ -25,23 +24,18 @@ impl<'a> DeleteGuildCommand<'a> {
         Self {
             application_id,
             command_id,
-            fut: None,
             guild_id,
             http,
         }
     }
 
-    fn start(&mut self) -> Result<(), Error> {
+    pub fn exec(self) -> ResponseFuture<EmptyBody> {
         let request = Request::from_route(Route::DeleteGuildCommand {
             application_id: self.application_id.0,
             command_id: self.command_id.0,
             guild_id: self.guild_id.0,
         });
 
-        self.fut.replace(Box::pin(self.http.verify(request)));
-
-        Ok(())
+        self.http.request(request)
     }
 }
-
-poll_req!(DeleteGuildCommand<'_>, ());
