@@ -1,7 +1,7 @@
 use crate::{
     client::Client,
-    error::Error,
-    request::{Pending, Request},
+    request::Request,
+    response::{marker::ListBody, ResponseFuture},
     routing::Route,
 };
 use twilight_model::{application::command::Command, id::ApplicationId};
@@ -9,28 +9,25 @@ use twilight_model::{application::command::Command, id::ApplicationId};
 /// Retrieve all global commands for an application.
 pub struct GetGlobalCommands<'a> {
     application_id: ApplicationId,
-    fut: Option<Pending<'a, Vec<Command>>>,
     http: &'a Client,
 }
 
 impl<'a> GetGlobalCommands<'a> {
-    pub(crate) fn new(http: &'a Client, application_id: ApplicationId) -> Self {
+    pub(crate) const fn new(http: &'a Client, application_id: ApplicationId) -> Self {
         Self {
             application_id,
-            fut: None,
             http,
         }
     }
 
-    fn start(&mut self) -> Result<(), Error> {
+    /// Execute the request, returning a future resolving to a [`Response`].
+    ///
+    /// [`Response`]: crate::response::Response
+    pub fn exec(self) -> ResponseFuture<ListBody<Command>> {
         let request = Request::from_route(Route::GetGlobalCommands {
             application_id: self.application_id.0,
         });
 
-        self.fut.replace(Box::pin(self.http.request(request)));
-
-        Ok(())
+        self.http.request(request)
     }
 }
-
-poll_req!(GetGlobalCommands<'_>, Vec<Command>);
