@@ -1,7 +1,8 @@
 use crate::{
     client::Client,
     error::Error,
-    request::{Pending, Request},
+    request::{Request, RequestBuilder},
+    response::{marker::ListBody, ResponseFuture},
     routing::Route,
 };
 use twilight_model::{application::command::Command, id::ApplicationId};
@@ -11,37 +12,39 @@ use twilight_model::{application::command::Command, id::ApplicationId};
 /// This method is idempotent: it can be used on every start, without being
 /// ratelimited if there aren't changes to the commands.
 pub struct SetGlobalCommands<'a> {
-    commands: Vec<Command>,
+    commands: &'a [Command],
     application_id: ApplicationId,
-    fut: Option<Pending<'a, ()>>,
     http: &'a Client,
 }
 
 impl<'a> SetGlobalCommands<'a> {
-    pub(crate) fn new(
+    pub(crate) const fn new(
         http: &'a Client,
         application_id: ApplicationId,
-        commands: Vec<Command>,
+        commands: &'a [Command],
     ) -> Self {
         Self {
             commands,
             application_id,
-            fut: None,
             http,
         }
     }
 
-    fn start(&mut self) -> Result<(), Error> {
-        let request = Request::builder(Route::SetGlobalCommands {
+    fn request(&self) -> Result<Request<'a>, Error> {
+        Request::builder(Route::SetGlobalCommands {
             application_id: self.application_id.0,
         })
-        .json(&self.commands)?;
+        .json(&self.commands)
+        .map(RequestBuilder::build)
+    }
 
-        self.fut
-            .replace(Box::pin(self.http.verify(request.build())));
-
-        Ok(())
+    /// Execute the request, returning a future resolving to a [`Response`].
+    ///
+    /// [`Response`]: crate::response::Response
+    pub fn exec(self) -> ResponseFuture<ListBody<Command>> {
+        match self.request() {
+            Ok(request) => self.http.request(request),
+            Err(source) => ResponseFuture::error(source),
+        }
     }
 }
-
-poll_req!(SetGlobalCommands<'_>, ());
