@@ -3,7 +3,8 @@
 use crate::{
     client::Client,
     error::Error as HttpError,
-    request::{validate, Form, NullableField, Pending, Request},
+    request::{self, validate, Form, NullableField, Request},
+    response::ResponseFuture,
     routing::Route,
 };
 use serde::Serialize;
@@ -51,16 +52,14 @@ impl UpdateOriginalResponseError {
 impl Display for UpdateOriginalResponseError {
     fn fmt(&self, f: &mut Formatter<'_>) -> FmtResult {
         match &self.kind {
-            UpdateOriginalResponseErrorType::ContentInvalid { .. } => {
+            UpdateOriginalResponseErrorType::ContentInvalid => {
                 f.write_str("message content is invalid")
             }
             UpdateOriginalResponseErrorType::EmbedTooLarge { .. } => {
                 f.write_str("length of one of the embeds is too large")
             }
-            UpdateOriginalResponseErrorType::TooManyEmbeds { embeds } => {
-                Display::fmt(&embeds.len(), f)?;
-
-                f.write_str(" embeds were provided, but only 10 may be provided")
+            UpdateOriginalResponseErrorType::TooManyEmbeds => {
+                f.write_str("more than 10 embeds were provided")
             }
         }
     }
@@ -79,42 +78,33 @@ impl Error for UpdateOriginalResponseError {
 #[non_exhaustive]
 pub enum UpdateOriginalResponseErrorType {
     /// Content is over 2000 UTF-16 characters.
-    ContentInvalid {
-        /// Provided content.
-        content: String,
-    },
+    ContentInvalid,
     /// Length of one of the embeds is over 6000 characters.
     EmbedTooLarge {
-        /// Provided embeds.
-        embeds: Vec<Embed>,
         /// Index of the embed that was too large.
         ///
-        /// This can be used to index into [`embeds`] to retrieve the bad embed.
-        ///
-        /// [`embeds`]: Self::EmbedTooLarge.embeds
+        /// This can be used to index into the provided embeds to retrieve the
+        /// invalid embed.
         index: usize,
     },
     /// Too many embeds were provided.
     ///
     /// A original response can have up to 10 embeds.
-    TooManyEmbeds {
-        /// Provided embeds.
-        embeds: Vec<Embed>,
-    },
+    TooManyEmbeds,
 }
 
-#[derive(Default, Serialize)]
-struct UpdateOriginalResponseFields {
+#[derive(Serialize)]
+struct UpdateOriginalResponseFields<'a> {
     #[serde(skip_serializing_if = "Option::is_none")]
     allowed_mentions: Option<AllowedMentions>,
-    #[serde(skip_serializing_if = "Vec::is_empty")]
-    attachments: Vec<Attachment>,
+    #[serde(skip_serializing_if = "request::slice_is_empty")]
+    attachments: &'a [Attachment],
     #[serde(skip_serializing_if = "Option::is_none")]
-    content: Option<NullableField<String>>,
+    content: Option<NullableField<&'a str>>,
     #[serde(skip_serializing_if = "Option::is_none")]
-    embeds: Option<NullableField<Vec<Embed>>>,
+    embeds: Option<NullableField<&'a [Embed]>>,
     #[serde(skip_serializing_if = "Option::is_none")]
-    payload_json: Option<Vec<u8>>,
+    payload_json: Option<&'a [u8]>,
 }
 
 /// Update the original response created by a interaction.
@@ -146,7 +136,8 @@ struct UpdateOriginalResponseFields {
 ///     // By creating a default set of allowed mentions, no entity can be
 ///     // mentioned.
 ///     .allowed_mentions(AllowedMentions::default())
-///     .content(Some("test <@3>".to_owned()))?
+///     .content(Some("test <@3>"))?
+///     .exec()
 ///     .await?;
 /// # Ok(()) }
 /// ```
@@ -154,32 +145,33 @@ struct UpdateOriginalResponseFields {
 /// [`DeleteOriginalResponse`]: super::DeleteOriginalResponse
 pub struct UpdateOriginalResponse<'a> {
     application_id: ApplicationId,
-    fields: UpdateOriginalResponseFields,
-    files: Vec<(String, Vec<u8>)>,
-    fut: Option<Pending<'a, Message>>,
+    fields: UpdateOriginalResponseFields<'a>,
+    files: &'a [(&'a str, &'a [u8])],
     http: &'a Client,
-    token: String,
+    token: &'a str,
 }
 
 impl<'a> UpdateOriginalResponse<'a> {
     /// Maximum number of embeds that a original response may have.
     pub const EMBED_COUNT_LIMIT: usize = 10;
 
-    pub(crate) fn new(
+    pub(crate) const fn new(
         http: &'a Client,
         application_id: ApplicationId,
-        interaction_token: impl Into<String>,
+        interaction_token: &'a str,
     ) -> Self {
         Self {
             application_id,
             fields: UpdateOriginalResponseFields {
-                allowed_mentions: http.default_allowed_mentions(),
-                ..UpdateOriginalResponseFields::default()
+                allowed_mentions: None,
+                attachments: &[],
+                content: None,
+                embeds: None,
+                payload_json: None,
             },
-            files: Vec::new(),
-            fut: None,
+            files: &[],
             http,
-            token: interaction_token.into(),
+            token: interaction_token,
         }
     }
 
@@ -190,25 +182,12 @@ impl<'a> UpdateOriginalResponse<'a> {
         self
     }
 
-    /// Specify an attachment already present in the target message to keep.
-    ///
-    /// If called, all unspecified attachments will be removed from the message.
-    /// If not called, all attachments will be kept.
-    #[deprecated(since = "0.5.5", note = "will be removed in favor of `attachments`")]
-    pub fn attachment(mut self, attachment: Attachment) -> Self {
-        self.fields.attachments.push(attachment);
-
-        self
-    }
-
     /// Specify multiple attachments already present in the target message to keep.
     ///
     /// If called, all unspecified attachments will be removed from the message.
     /// If not called, all attachments will be kept.
-    pub fn attachments(mut self, attachments: impl IntoIterator<Item = Attachment>) -> Self {
-        self.fields
-            .attachments
-            .extend(attachments.into_iter().collect::<Vec<Attachment>>());
+    pub const fn attachments(mut self, attachments: &'a [Attachment]) -> Self {
+        self.fields.attachments = attachments;
 
         self
     }
@@ -226,21 +205,20 @@ impl<'a> UpdateOriginalResponse<'a> {
     ///
     /// Returns an [`UpdateOriginalResponseErrorType::ContentInvalid`] error type if
     /// the content length is too long.
-    pub fn content(mut self, content: Option<String>) -> Result<Self, UpdateOriginalResponseError> {
-        if let Some(content_ref) = content.as_ref() {
+    pub fn content(
+        mut self,
+        content: Option<&'a str>,
+    ) -> Result<Self, UpdateOriginalResponseError> {
+        if let Some(content_ref) = content {
             if !validate::content_limit(content_ref) {
                 return Err(UpdateOriginalResponseError {
-                    kind: UpdateOriginalResponseErrorType::ContentInvalid {
-                        content: content.expect("content is known to be some"),
-                    },
+                    kind: UpdateOriginalResponseErrorType::ContentInvalid,
                     source: None,
                 });
             }
         }
 
-        self.fields
-            .content
-            .replace(NullableField::from_option(content));
+        self.fields.content = Some(NullableField(content));
 
         Ok(self)
     }
@@ -280,7 +258,8 @@ impl<'a> UpdateOriginalResponse<'a> {
     ///     .build()?;
     ///
     /// client.update_interaction_original("token")?
-    ///     .embeds(Some(vec![embed]))?
+    ///     .embeds(Some(&[embed]))?
+    ///     .exec()
     ///     .await?;
     /// # Ok(()) }
     /// ```
@@ -297,58 +276,34 @@ impl<'a> UpdateOriginalResponse<'a> {
     /// [`EMBED_COUNT_LIMIT`]: Self::EMBED_COUNT_LIMIT
     pub fn embeds(
         mut self,
-        embeds: Option<Vec<Embed>>,
+        embeds: Option<&'a [Embed]>,
     ) -> Result<Self, UpdateOriginalResponseError> {
-        if let Some(embeds_present) = embeds.as_deref() {
+        if let Some(embeds_present) = embeds {
             if embeds_present.len() > Self::EMBED_COUNT_LIMIT {
                 return Err(UpdateOriginalResponseError {
-                    kind: UpdateOriginalResponseErrorType::TooManyEmbeds {
-                        embeds: embeds.expect("embeds are known to be present"),
-                    },
+                    kind: UpdateOriginalResponseErrorType::TooManyEmbeds,
                     source: None,
                 });
             }
 
             for (idx, embed) in embeds_present.iter().enumerate() {
-                if let Err(source) = validate::embed(&embed) {
+                if let Err(source) = validate::embed(embed) {
                     return Err(UpdateOriginalResponseError {
-                        kind: UpdateOriginalResponseErrorType::EmbedTooLarge {
-                            embeds: embeds.expect("embeds are known to be present"),
-                            index: idx,
-                        },
+                        kind: UpdateOriginalResponseErrorType::EmbedTooLarge { index: idx },
                         source: Some(Box::new(source)),
                     });
                 }
             }
         }
 
-        self.fields
-            .embeds
-            .replace(NullableField::from_option(embeds));
+        self.fields.embeds = Some(NullableField(embeds));
 
         Ok(self)
     }
 
-    /// Attach a file to the original response.
-    ///
-    /// This method is repeatable.
-    #[deprecated(since = "0.5.5", note = "will be removed in favor of `files`")]
-    pub fn file(mut self, name: impl Into<String>, file: impl Into<Vec<u8>>) -> Self {
-        self.files.push((name.into(), file.into()));
-
-        self
-    }
-
     /// Attach multiple files to the original response.
-    pub fn files<N: Into<String>, F: Into<Vec<u8>>>(
-        mut self,
-        files: impl IntoIterator<Item = (N, F)>,
-    ) -> Self {
-        self.files.extend(
-            files
-                .into_iter()
-                .map(|(name, file)| (name.into(), file.into())),
-        );
+    pub const fn files(mut self, files: &'a [(&'a str, &'a [u8])]) -> Self {
+        self.files = files;
 
         self
     }
@@ -356,52 +311,60 @@ impl<'a> UpdateOriginalResponse<'a> {
     /// JSON encoded body of any additional request fields.
     ///
     /// If this method is called, all other fields are ignored, except for
-    /// [`file`]. See [Discord Docs/Create Message] and
+    /// [`files`]. See [Discord Docs/Create Message] and
     /// [`CreateFollowupMessage::payload_json`].
     ///
-    /// [`file`]: Self::file
+    /// [`files`]: Self::files
     /// [`CreateFollowupMessage::payload_json`]: super::CreateFollowupMessage::payload_json
     /// [Discord Docs/Create Message]: https://discord.com/developers/docs/resources/channel#create-message-params
-    pub fn payload_json(mut self, payload_json: impl Into<Vec<u8>>) -> Self {
-        self.fields.payload_json.replace(payload_json.into());
+    pub const fn payload_json(mut self, payload_json: &'a [u8]) -> Self {
+        self.fields.payload_json = Some(payload_json);
 
         self
     }
 
-    fn request(&mut self) -> Result<Request, HttpError> {
+    // `self` needs to be consumed and the client returned due to parameters
+    // being consumed in request construction.
+    fn request(&mut self) -> Result<Request<'a>, HttpError> {
         let mut request = Request::builder(Route::UpdateInteractionOriginal {
             application_id: self.application_id.0,
-            interaction_token: self.token.clone(),
+            interaction_token: self.token,
         });
 
         if !self.files.is_empty() || self.fields.payload_json.is_some() {
             let mut form = Form::new();
 
-            for (index, (name, file)) in self.files.drain(..).enumerate() {
-                form.file(format!("{}", index).as_bytes(), name.as_bytes(), &file);
+            for (index, (name, file)) in self.files.iter().enumerate() {
+                form.file(index.to_be_bytes().as_ref(), name.as_bytes(), file);
             }
 
             if let Some(payload_json) = &self.fields.payload_json {
-                form.payload_json(&payload_json);
+                form.payload_json(payload_json);
             } else {
+                if self.fields.allowed_mentions.is_none() {
+                    self.fields.allowed_mentions = self.http.default_allowed_mentions();
+                }
+
                 let body = crate::json::to_vec(&self.fields).map_err(HttpError::json)?;
                 form.payload_json(&body);
             }
 
             request = request.form(form);
         } else {
+            if self.fields.allowed_mentions.is_none() {
+                self.fields.allowed_mentions = self.http.default_allowed_mentions();
+            }
+
             request = request.json(&self.fields)?;
         }
 
         Ok(request.build())
     }
 
-    fn start(&mut self) -> Result<(), HttpError> {
-        let request = self.request()?;
-        self.fut.replace(Box::pin(self.http.request(request)));
-
-        Ok(())
+    pub fn exec(mut self) -> ResponseFuture<Message> {
+        match self.request() {
+            Ok(request) => self.http.request(request),
+            Err(source) => ResponseFuture::error(source),
+        }
     }
 }
-
-poll_req!(UpdateOriginalResponse<'_>, Message);
