@@ -1,7 +1,7 @@
 use crate::{
     client::Client,
-    error::Error,
-    request::{Pending, Request},
+    request::Request,
+    response::{marker::EmptyBody, ResponseFuture},
     routing::Route,
 };
 use twilight_model::id::ChannelId;
@@ -9,28 +9,22 @@ use twilight_model::id::ChannelId;
 /// Fire a Typing Start event in the channel.
 pub struct CreateTypingTrigger<'a> {
     channel_id: ChannelId,
-    fut: Option<Pending<'a, ()>>,
     http: &'a Client,
 }
 
 impl<'a> CreateTypingTrigger<'a> {
-    pub(crate) fn new(http: &'a Client, channel_id: ChannelId) -> Self {
-        Self {
-            channel_id,
-            fut: None,
-            http,
-        }
+    pub(crate) const fn new(http: &'a Client, channel_id: ChannelId) -> Self {
+        Self { channel_id, http }
     }
 
-    fn start(&mut self) -> Result<(), Error> {
+    /// Execute the request, returning a future resolving to a [`Response`].
+    ///
+    /// [`Response`]: crate::response::Response
+    pub fn exec(self) -> ResponseFuture<EmptyBody> {
         let request = Request::from_route(Route::CreateTypingTrigger {
             channel_id: self.channel_id.0,
         });
 
-        self.fut.replace(Box::pin(self.http.verify(request)));
-
-        Ok(())
+        self.http.request(request)
     }
 }
-
-poll_req!(CreateTypingTrigger<'_>, ());
