@@ -1,9 +1,4 @@
-use crate::{
-    client::Client,
-    error::Error,
-    request::{PendingOption, Request},
-    routing::Route,
-};
+use crate::{client::Client, request::Request, response::ResponseFuture, routing::Route};
 use twilight_model::{channel::Channel, id::ChannelId};
 
 /// Get a channel by its ID.
@@ -18,37 +13,31 @@ use twilight_model::{channel::Channel, id::ChannelId};
 ///
 /// # #[tokio::main]
 /// # async fn main() -> Result<(), Box<dyn std::error::Error>> {
-/// let client = Client::new("my token");
+/// let client = Client::new("my token".to_owned());
 ///
 /// let channel_id = ChannelId(100);
 ///
-/// let channel = client.channel(channel_id).await?;
+/// let channel = client.channel(channel_id).exec().await?;
 /// # Ok(()) }
 /// ```
 pub struct GetChannel<'a> {
     channel_id: ChannelId,
-    fut: Option<PendingOption<'a>>,
     http: &'a Client,
 }
 
 impl<'a> GetChannel<'a> {
-    pub(crate) fn new(http: &'a Client, channel_id: ChannelId) -> Self {
-        Self {
-            channel_id,
-            fut: None,
-            http,
-        }
+    pub(crate) const fn new(http: &'a Client, channel_id: ChannelId) -> Self {
+        Self { channel_id, http }
     }
 
-    fn start(&mut self) -> Result<(), Error> {
+    /// Execute the request, returning a future resolving to a [`Response`].
+    ///
+    /// [`Response`]: crate::response::Response
+    pub fn exec(self) -> ResponseFuture<Channel> {
         let request = Request::from_route(Route::GetChannel {
             channel_id: self.channel_id.0,
         });
 
-        self.fut.replace(Box::pin(self.http.request_bytes(request)));
-
-        Ok(())
+        self.http.request(request)
     }
 }
-
-poll_req!(opt, GetChannel<'_>, Channel);
