@@ -1,7 +1,7 @@
 use crate::{
     client::Client,
-    error::Error as HttpError,
-    request::{self, validate, AuditLogReason, AuditLogReasonError, Pending, Request},
+    request::{self, validate, AuditLogReason, AuditLogReasonError, Request},
+    response::ResponseFuture,
     routing::Route,
 };
 use serde::Serialize;
@@ -57,18 +57,12 @@ impl Error for CreateInviteError {}
 #[non_exhaustive]
 pub enum CreateInviteErrorType {
     /// Configured maximum age is over 604800.
-    MaxAgeTooOld {
-        /// Provided maximum age.
-        provided: u64,
-    },
+    MaxAgeTooOld,
     /// Configured maximum uses is over 100.
-    MaxUsesTooLarge {
-        /// Provided maximum uses.
-        provided: u64,
-    },
+    MaxUsesTooLarge,
 }
 
-#[derive(Default, Serialize)]
+#[derive(Serialize)]
 struct CreateInviteFields {
     #[serde(skip_serializing_if = "Option::is_none")]
     max_age: Option<u64>,
@@ -79,7 +73,7 @@ struct CreateInviteFields {
     #[serde(skip_serializing_if = "Option::is_none")]
     target_application_id: Option<ApplicationId>,
     #[serde(skip_serializing_if = "Option::is_none")]
-    target_user_id: Option<String>,
+    target_user_id: Option<UserId>,
     #[serde(skip_serializing_if = "Option::is_none")]
     target_type: Option<TargetType>,
     #[serde(skip_serializing_if = "Option::is_none")]
@@ -98,12 +92,13 @@ struct CreateInviteFields {
 ///
 /// # #[tokio::main]
 /// # async fn main() -> Result<(), Box<dyn std::error::Error>> {
-/// let client = Client::new("my token");
+/// let client = Client::new("my token".to_owned());
 ///
 /// let channel_id = ChannelId(123);
 /// let invite = client
 ///     .create_invite(channel_id)
 ///     .max_uses(3)?
+///     .exec()
 ///     .await?;
 /// # Ok(()) }
 /// ```
@@ -112,17 +107,23 @@ struct CreateInviteFields {
 pub struct CreateInvite<'a> {
     channel_id: ChannelId,
     fields: CreateInviteFields,
-    fut: Option<Pending<'a, Invite>>,
     http: &'a Client,
-    reason: Option<String>,
+    reason: Option<&'a str>,
 }
 
 impl<'a> CreateInvite<'a> {
-    pub(crate) fn new(http: &'a Client, channel_id: ChannelId) -> Self {
+    pub(crate) const fn new(http: &'a Client, channel_id: ChannelId) -> Self {
         Self {
             channel_id,
-            fields: CreateInviteFields::default(),
-            fut: None,
+            fields: CreateInviteFields {
+                max_age: None,
+                max_uses: None,
+                temporary: None,
+                target_application_id: None,
+                target_user_id: None,
+                target_type: None,
+                unique: None,
+            },
             http,
             reason: None,
         }
@@ -146,19 +147,24 @@ impl<'a> CreateInvite<'a> {
     /// # #[tokio::main]
     /// # async fn main() -> Result<(), Box<dyn std::error::Error>> {
     /// let client = Client::new(env::var("DISCORD_TOKEN")?);
-    /// let invite = client.create_invite(ChannelId(1)).max_age(60 * 60)?.await?;
+    /// let invite = client.create_invite(ChannelId(1))
+    ///     .max_age(60 * 60)?
+    ///     .exec()
+    ///     .await?
+    ///     .model()
+    ///     .await?;
     ///
     /// println!("invite code: {}", invite.code);
     /// # Ok(()) }
     /// ```
-    pub fn max_age(mut self, max_age: u64) -> Result<Self, CreateInviteError> {
+    pub const fn max_age(mut self, max_age: u64) -> Result<Self, CreateInviteError> {
         if !validate::invite_max_age(max_age) {
             return Err(CreateInviteError {
-                kind: CreateInviteErrorType::MaxAgeTooOld { provided: max_age },
+                kind: CreateInviteErrorType::MaxAgeTooOld,
             });
         }
 
-        self.fields.max_age.replace(max_age);
+        self.fields.max_age = Some(max_age);
 
         Ok(self)
     }
@@ -179,19 +185,24 @@ impl<'a> CreateInvite<'a> {
     /// # #[tokio::main]
     /// # async fn main() -> Result<(), Box<dyn std::error::Error>> {
     /// let client = Client::new(env::var("DISCORD_TOKEN")?);
-    /// let invite = client.create_invite(ChannelId(1)).max_uses(5)?.await?;
+    /// let invite = client.create_invite(ChannelId(1))
+    ///     .max_uses(5)?
+    ///     .exec()
+    ///     .await?
+    ///     .model()
+    ///     .await?;
     ///
     /// println!("invite code: {}", invite.code);
     /// # Ok(()) }
     /// ```
-    pub fn max_uses(mut self, max_uses: u64) -> Result<Self, CreateInviteError> {
+    pub const fn max_uses(mut self, max_uses: u64) -> Result<Self, CreateInviteError> {
         if !validate::invite_max_uses(max_uses) {
             return Err(CreateInviteError {
-                kind: CreateInviteErrorType::MaxUsesTooLarge { provided: max_uses },
+                kind: CreateInviteErrorType::MaxUsesTooLarge,
             });
         }
 
-        self.fields.max_uses.replace(max_uses);
+        self.fields.max_uses = Some(max_uses);
 
         Ok(self)
     }
@@ -201,26 +212,22 @@ impl<'a> CreateInvite<'a> {
     /// This only works if [`target_type`] is set to [`TargetType::EmbeddedApplication`].
     ///
     /// [`target_type`]: Self::target_type
-    pub fn target_application_id(mut self, target_application_id: ApplicationId) -> Self {
-        self.fields
-            .target_application_id
-            .replace(target_application_id);
+    pub const fn target_application_id(mut self, target_application_id: ApplicationId) -> Self {
+        self.fields.target_application_id = Some(target_application_id);
 
         self
     }
 
     /// Set the target user id for this invite.
-    pub fn target_user_id(mut self, target_user_id: UserId) -> Self {
-        self.fields
-            .target_user_id
-            .replace(target_user_id.0.to_string());
+    pub const fn target_user_id(mut self, target_user_id: UserId) -> Self {
+        self.fields.target_user_id = Some(target_user_id);
 
         self
     }
 
     /// Set the target type for this invite.
-    pub fn target_type(mut self, target_type: TargetType) -> Self {
-        self.fields.target_type.replace(target_type);
+    pub const fn target_type(mut self, target_type: TargetType) -> Self {
+        self.fields.target_type = Some(target_type);
 
         self
     }
@@ -228,8 +235,8 @@ impl<'a> CreateInvite<'a> {
     /// Specify true if the invite should grant temporary membership.
     ///
     /// Defaults to false.
-    pub fn temporary(mut self, temporary: bool) -> Self {
-        self.fields.temporary.replace(temporary);
+    pub const fn temporary(mut self, temporary: bool) -> Self {
+        self.fields.temporary = Some(temporary);
 
         self
     }
@@ -240,40 +247,46 @@ impl<'a> CreateInvite<'a> {
     /// invites). Refer to [the discord docs] for more information.
     ///
     /// [the discord docs]: https://discord.com/developers/docs/resources/channel#create-channel-invite
-    pub fn unique(mut self, unique: bool) -> Self {
-        self.fields.unique.replace(unique);
+    pub const fn unique(mut self, unique: bool) -> Self {
+        self.fields.unique = Some(unique);
 
         self
     }
 
-    fn start(&mut self) -> Result<(), HttpError> {
+    /// Execute the request, returning a future resolving to a [`Response`].
+    ///
+    /// [`Response`]: crate::response::Response
+    pub fn exec(self) -> ResponseFuture<Invite> {
         let mut request = Request::builder(Route::CreateInvite {
             channel_id: self.channel_id.0,
-        })
-        .json(&self.fields)?;
+        });
+
+        request = match request.json(&self.fields) {
+            Ok(request) => request,
+            Err(source) => return ResponseFuture::error(source),
+        };
 
         if let Some(reason) = &self.reason {
-            request = request.headers(request::audit_header(reason)?);
+            let header = match request::audit_header(reason) {
+                Ok(header) => header,
+                Err(source) => return ResponseFuture::error(source),
+            };
+
+            request = request.headers(header);
         }
 
-        self.fut
-            .replace(Box::pin(self.http.request(request.build())));
-
-        Ok(())
+        self.http.request(request.build())
     }
 }
 
-impl<'a> AuditLogReason for CreateInvite<'a> {
-    fn reason(mut self, reason: impl Into<String>) -> Result<Self, AuditLogReasonError> {
-        self.reason
-            .replace(AuditLogReasonError::validate(reason.into())?);
+impl<'a> AuditLogReason<'a> for CreateInvite<'a> {
+    fn reason(mut self, reason: &'a str) -> Result<Self, AuditLogReasonError> {
+        self.reason.replace(AuditLogReasonError::validate(reason)?);
 
         Ok(self)
     }
 }
 
-poll_req!(CreateInvite<'_>, Invite);
-
 #[cfg(test)]
 mod tests {
     use super::CreateInvite;
@@ -283,7 +296,7 @@ mod tests {
 
     #[test]
     fn test_max_age() -> Result<(), Box<dyn Error>> {
-        let client = Client::new("foo");
+        let client = Client::new("foo".to_owned());
         let mut builder = CreateInvite::new(&client, ChannelId(1)).max_age(0)?;
         assert_eq!(Some(0), builder.fields.max_age);
         builder = builder.max_age(604_800)?;
@@ -295,7 +308,7 @@ mod tests {
 
     #[test]
     fn test_max_uses() -> Result<(), Box<dyn Error>> {
-        let client = Client::new("foo");
+        let client = Client::new("foo".to_owned());
         let mut builder = CreateInvite::new(&client, ChannelId(1)).max_uses(0)?;
         assert_eq!(Some(0), builder.fields.max_uses);
         builder = builder.max_uses(100)?;
