@@ -1,8 +1,8 @@
 use super::RequestReactionType;
 use crate::{
     client::Client,
-    error::Error,
-    request::{Pending, Request},
+    request::Request,
+    response::{marker::EmptyBody, ResponseFuture},
     routing::Route,
 };
 use twilight_model::id::{ChannelId, MessageId};
@@ -20,88 +20,83 @@ use twilight_model::id::{ChannelId, MessageId};
 ///
 /// # #[tokio::main]
 /// # async fn main() -> Result<(), Box<dyn std::error::Error>> {
-/// let client = Client::new("my token");
+/// let client = Client::new("my token".to_owned());
 ///
 /// let channel_id = ChannelId(123);
 /// let message_id = MessageId(456);
-/// let emoji = RequestReactionType::Unicode { name: String::from("🌃") };
+/// let emoji = RequestReactionType::Unicode { name: "🌃" };
 ///
 /// let reaction = client
-///     .create_reaction(channel_id, message_id, emoji)
+///     .create_reaction(channel_id, message_id, &emoji)
+///     .exec()
 ///     .await?;
 /// # Ok(()) }
 /// ```
 pub struct CreateReaction<'a> {
     channel_id: ChannelId,
-    emoji: RequestReactionType,
-    fut: Option<Pending<'a, ()>>,
+    emoji: &'a RequestReactionType<'a>,
     http: &'a Client,
     message_id: MessageId,
 }
 
 impl<'a> CreateReaction<'a> {
-    pub(crate) fn new(
+    pub(crate) const fn new(
         http: &'a Client,
         channel_id: ChannelId,
         message_id: MessageId,
-        emoji: RequestReactionType,
+        emoji: &'a RequestReactionType<'a>,
     ) -> Self {
         Self {
             channel_id,
             emoji,
-            fut: None,
             http,
             message_id,
         }
     }
 
-    fn request(&self) -> Request {
+    const fn request(&self) -> Request<'a> {
         Request::from_route(Route::CreateReaction {
             channel_id: self.channel_id.0,
-            emoji: self.emoji.display().to_string(),
+            emoji: self.emoji,
             message_id: self.message_id.0,
         })
     }
 
-    fn start(&mut self) -> Result<(), Error> {
-        let request = self.request();
-
-        self.fut.replace(Box::pin(self.http.verify(request)));
-
-        Ok(())
+    /// Execute the request, returning a future resolving to a [`Response`].
+    ///
+    /// [`Response`]: crate::response::Response
+    pub fn exec(self) -> ResponseFuture<EmptyBody> {
+        self.http.request(self.request())
     }
 }
 
-poll_req!(CreateReaction<'_>, ());
-
 #[cfg(test)]
 mod tests {
+    #![allow(clippy::non_ascii_literal)]
+
     use super::CreateReaction;
     use crate::{
         request::{channel::reaction::RequestReactionType, Request},
         routing::Route,
         Client,
     };
-    use percent_encoding::{utf8_percent_encode, NON_ALPHANUMERIC};
     use twilight_model::id::{ChannelId, MessageId};
 
     #[test]
     fn test_request() {
-        let client = Client::new("foo");
+        let client = Client::new("foo".to_owned());
 
-        let emoji = RequestReactionType::Unicode {
-            name: String::from("\u{1f303}"),
-        };
+        let emoji = RequestReactionType::Unicode { name: "🌃" };
 
-        let builder = CreateReaction::new(&client, ChannelId(123), MessageId(456), emoji);
+        let builder = CreateReaction::new(&client, ChannelId(123), MessageId(456), &emoji);
         let actual = builder.request();
 
         let expected = Request::from_route(Route::CreateReaction {
             channel_id: 123,
-            emoji: utf8_percent_encode("\u{1f303}", NON_ALPHANUMERIC).to_string(),
+            emoji: &RequestReactionType::Unicode { name: "🌃" },
             message_id: 456,
         });
 
-        assert_eq!(actual.path_str, expected.path_str);
+        assert_eq!(actual.route, expected.route);
     }
 }
