@@ -1,7 +1,7 @@
 use crate::{
     client::Client,
-    error::Error,
-    request::{Pending, Request},
+    request::Request,
+    response::{marker::EmptyBody, ResponseFuture},
     routing::Route,
 };
 use twilight_model::id::{ChannelId, MessageId};
@@ -9,31 +9,32 @@ use twilight_model::id::{ChannelId, MessageId};
 /// Delete all reactions by all users on a message.
 pub struct DeleteAllReactions<'a> {
     channel_id: ChannelId,
-    fut: Option<Pending<'a, ()>>,
     http: &'a Client,
     message_id: MessageId,
 }
 
 impl<'a> DeleteAllReactions<'a> {
-    pub(crate) fn new(http: &'a Client, channel_id: ChannelId, message_id: MessageId) -> Self {
+    pub(crate) const fn new(
+        http: &'a Client,
+        channel_id: ChannelId,
+        message_id: MessageId,
+    ) -> Self {
         Self {
             channel_id,
-            fut: None,
             http,
             message_id,
         }
     }
 
-    fn start(&mut self) -> Result<(), Error> {
+    /// Execute the request, returning a future resolving to a [`Response`].
+    ///
+    /// [`Response`]: crate::response::Response
+    pub fn exec(self) -> ResponseFuture<EmptyBody> {
         let request = Request::from_route(Route::DeleteMessageReactions {
             channel_id: self.channel_id.0,
             message_id: self.message_id.0,
         });
 
-        self.fut.replace(Box::pin(self.http.verify(request)));
-
-        Ok(())
+        self.http.request(request)
     }
 }
-
-poll_req!(DeleteAllReactions<'_>, ());
