@@ -1,9 +1,8 @@
 use crate::{
     client::Client,
     error::Error as HttpError,
-    request::{
-        self, validate, AuditLogReason, AuditLogReasonError, NullableField, Pending, Request,
-    },
+    request::{self, validate, AuditLogReason, AuditLogReasonError, NullableField, Request},
+    response::ResponseFuture,
     routing::Route,
 };
 use serde::Serialize;
@@ -46,13 +45,11 @@ impl UpdateChannelError {
 impl Display for UpdateChannelError {
     fn fmt(&self, f: &mut Formatter<'_>) -> FmtResult {
         match &self.kind {
-            UpdateChannelErrorType::NameInvalid { .. } => {
-                f.write_str("the length of the name is invalid")
-            }
-            UpdateChannelErrorType::RateLimitPerUserInvalid { .. } => {
+            UpdateChannelErrorType::NameInvalid => f.write_str("the length of the name is invalid"),
+            UpdateChannelErrorType::RateLimitPerUserInvalid => {
                 f.write_str("the rate limit per user is invalid")
             }
-            UpdateChannelErrorType::TopicInvalid { .. } => f.write_str("the topic is invalid"),
+            UpdateChannelErrorType::TopicInvalid => f.write_str("the topic is invalid"),
         }
     }
 }
@@ -62,44 +59,35 @@ impl Error for UpdateChannelError {}
 #[derive(Debug)]
 #[non_exhaustive]
 pub enum UpdateChannelErrorType {
-    /// The length of the name is either fewer than 1 UTF-16 character or more
-    /// than 100 UTF-16 characters.
-    NameInvalid {
-        /// Provided name.
-        name: String,
-    },
+    /// The length of the name is either fewer than 1 UTF-16 characters or
+    /// more than 100 UTF-16 characters.
+    NameInvalid,
     /// The seconds of the rate limit per user is more than 21600.
-    RateLimitPerUserInvalid {
-        /// Provided ratelimit is invalid.
-        rate_limit_per_user: u64,
-    },
+    RateLimitPerUserInvalid,
     /// The length of the topic is more than 1024 UTF-16 characters.
-    TopicInvalid {
-        /// Provided topic.
-        topic: String,
-    },
+    TopicInvalid,
 }
 
 // The Discord API doesn't require the `name` and `kind` fields to be present,
 // but it does require them to be non-null.
-#[derive(Default, Serialize)]
-struct UpdateChannelFields {
+#[derive(Serialize)]
+struct UpdateChannelFields<'a> {
     #[serde(skip_serializing_if = "Option::is_none")]
     bitrate: Option<u64>,
     #[serde(skip_serializing_if = "Option::is_none")]
-    name: Option<String>,
+    name: Option<&'a str>,
     #[serde(skip_serializing_if = "Option::is_none")]
     nsfw: Option<bool>,
     #[serde(skip_serializing_if = "Option::is_none")]
     parent_id: Option<NullableField<ChannelId>>,
     #[serde(skip_serializing_if = "Option::is_none")]
-    permission_overwrites: Option<Vec<PermissionOverwrite>>,
+    permission_overwrites: Option<&'a [PermissionOverwrite]>,
     #[serde(skip_serializing_if = "Option::is_none")]
     position: Option<u64>,
     #[serde(skip_serializing_if = "Option::is_none")]
     rate_limit_per_user: Option<u64>,
     #[serde(skip_serializing_if = "Option::is_none")]
-    topic: Option<String>,
+    topic: Option<&'a str>,
     #[serde(skip_serializing_if = "Option::is_none")]
     user_limit: Option<u64>,
     #[serde(skip_serializing_if = "Option::is_none")]
@@ -115,26 +103,36 @@ struct UpdateChannelFields {
 /// and the maximum is 100 UTF-16 characters.
 pub struct UpdateChannel<'a> {
     channel_id: ChannelId,
-    fields: UpdateChannelFields,
-    fut: Option<Pending<'a, Channel>>,
+    fields: UpdateChannelFields<'a>,
     http: &'a Client,
-    reason: Option<String>,
+    reason: Option<&'a str>,
 }
 
 impl<'a> UpdateChannel<'a> {
-    pub(crate) fn new(http: &'a Client, channel_id: ChannelId) -> Self {
+    pub(crate) const fn new(http: &'a Client, channel_id: ChannelId) -> Self {
         Self {
             channel_id,
-            fields: UpdateChannelFields::default(),
-            fut: None,
+            fields: UpdateChannelFields {
+                bitrate: None,
+                name: None,
+                nsfw: None,
+                parent_id: None,
+                permission_overwrites: None,
+                position: None,
+                rate_limit_per_user: None,
+                topic: None,
+                user_limit: None,
+                video_quality_mode: None,
+                kind: None,
+            },
             http,
             reason: None,
         }
     }
 
     /// Set the bitrate of the channel. Applicable to voice channels only.
-    pub fn bitrate(mut self, bitrate: u64) -> Self {
-        self.fields.bitrate.replace(bitrate);
+    pub const fn bitrate(mut self, bitrate: u64) -> Self {
+        self.fields.bitrate = Some(bitrate);
 
         self
     }
@@ -148,50 +146,40 @@ impl<'a> UpdateChannel<'a> {
     ///
     /// Returns an [`UpdateChannelErrorType::NameInvalid`] error type if the name
     /// length is too short or too long.
-    pub fn name(self, name: impl Into<String>) -> Result<Self, UpdateChannelError> {
-        self._name(name.into())
-    }
-
-    fn _name(mut self, name: String) -> Result<Self, UpdateChannelError> {
-        if !validate::channel_name(&name) {
+    pub fn name(mut self, name: &'a str) -> Result<Self, UpdateChannelError> {
+        if !validate::channel_name(name) {
             return Err(UpdateChannelError {
-                kind: UpdateChannelErrorType::NameInvalid { name },
+                kind: UpdateChannelErrorType::NameInvalid,
             });
         }
 
-        self.fields.name.replace(name);
+        self.fields.name = Some(name);
 
         Ok(self)
     }
 
     /// Set whether the channel is marked as NSFW.
-    pub fn nsfw(mut self, nsfw: bool) -> Self {
-        self.fields.nsfw.replace(nsfw);
+    pub const fn nsfw(mut self, nsfw: bool) -> Self {
+        self.fields.nsfw = Some(nsfw);
 
         self
     }
 
     /// If this is specified, and the parent ID is a `ChannelType::CategoryChannel`, move this
     /// channel to a child of the category channel.
-    pub fn parent_id(mut self, parent_id: impl Into<Option<ChannelId>>) -> Self {
-        let parent_id = parent_id.into();
-
-        self.fields
-            .parent_id
-            .replace(NullableField::from_option(parent_id));
+    pub const fn parent_id(mut self, parent_id: Option<ChannelId>) -> Self {
+        self.fields.parent_id = Some(NullableField(parent_id));
 
         self
     }
 
     /// Set the permission overwrites of a channel. This will overwrite all permissions that the
     /// channel currently has, so use with caution!
-    pub fn permission_overwrites(
+    pub const fn permission_overwrites(
         mut self,
-        permission_overwrites: Vec<PermissionOverwrite>,
+        permission_overwrites: &'a [PermissionOverwrite],
     ) -> Self {
-        self.fields
-            .permission_overwrites
-            .replace(permission_overwrites);
+        self.fields.permission_overwrites = Some(permission_overwrites);
 
         self
     }
@@ -200,8 +188,8 @@ impl<'a> UpdateChannel<'a> {
     ///
     /// Positions are numerical and zero-indexed. If you place a channel at position 2, channels
     /// 2-n will shift down one position and the initial channel will take its place.
-    pub fn position(mut self, position: u64) -> Self {
-        self.fields.position.replace(position);
+    pub const fn position(mut self, position: u64) -> Self {
+        self.fields.position = Some(position);
 
         self
     }
@@ -218,19 +206,17 @@ impl<'a> UpdateChannel<'a> {
     /// type if the amount is greater than 21600.
     ///
     /// [the discord docs]: https://discordapp.com/developers/docs/resources/channel#channel-object-channel-structure>
-    pub fn rate_limit_per_user(
+    pub const fn rate_limit_per_user(
         mut self,
         rate_limit_per_user: u64,
     ) -> Result<Self, UpdateChannelError> {
         if rate_limit_per_user > 21600 {
             return Err(UpdateChannelError {
-                kind: UpdateChannelErrorType::RateLimitPerUserInvalid {
-                    rate_limit_per_user,
-                },
+                kind: UpdateChannelErrorType::RateLimitPerUserInvalid,
             });
         }
 
-        self.fields.rate_limit_per_user.replace(rate_limit_per_user);
+        self.fields.rate_limit_per_user = Some(rate_limit_per_user);
 
         Ok(self)
     }
@@ -245,14 +231,10 @@ impl<'a> UpdateChannel<'a> {
     /// length is too long.
     ///
     /// [the discord docs]: https://discordapp.com/developers/docs/resources/channel#channel-object-channel-structure
-    pub fn topic(self, topic: impl Into<String>) -> Result<Self, UpdateChannelError> {
-        self._topic(topic.into())
-    }
-
-    fn _topic(mut self, topic: String) -> Result<Self, UpdateChannelError> {
+    pub fn topic(mut self, topic: &'a str) -> Result<Self, UpdateChannelError> {
         if topic.chars().count() > 1024 {
             return Err(UpdateChannelError {
-                kind: UpdateChannelErrorType::TopicInvalid { topic },
+                kind: UpdateChannelErrorType::TopicInvalid,
             });
         }
 
@@ -267,15 +249,15 @@ impl<'a> UpdateChannel<'a> {
     /// discord docs] for more details.
     ///
     /// [the discord docs]: https://discord.com/developers/docs/resources/channel#modify-channel-json-params
-    pub fn user_limit(mut self, user_limit: u64) -> Self {
-        self.fields.user_limit.replace(user_limit);
+    pub const fn user_limit(mut self, user_limit: u64) -> Self {
+        self.fields.user_limit = Some(user_limit);
 
         self
     }
 
     /// Set the [`VideoQualityMode`] for the voice channel.
-    pub fn video_quality_mode(mut self, video_quality_mode: VideoQualityMode) -> Self {
-        self.fields.video_quality_mode.replace(video_quality_mode);
+    pub const fn video_quality_mode(mut self, video_quality_mode: VideoQualityMode) -> Self {
+        self.fields.video_quality_mode = Some(video_quality_mode);
 
         self
     }
@@ -287,13 +269,13 @@ impl<'a> UpdateChannel<'a> {
     /// details.
     ///
     /// [the discord docs]: https://discord.com/developers/docs/resources/channel#modify-channel-json-params
-    pub fn kind(mut self, kind: ChannelType) -> Self {
-        self.fields.kind.replace(kind);
+    pub const fn kind(mut self, kind: ChannelType) -> Self {
+        self.fields.kind = Some(kind);
 
         self
     }
 
-    fn start(&mut self) -> Result<(), HttpError> {
+    fn request(&self) -> Result<Request<'a>, HttpError> {
         let mut request = Request::builder(Route::UpdateChannel {
             channel_id: self.channel_id.0,
         })
@@ -303,20 +285,24 @@ impl<'a> UpdateChannel<'a> {
             request = request.headers(request::audit_header(reason)?);
         }
 
-        self.fut
-            .replace(Box::pin(self.http.request(request.build())));
+        Ok(request.build())
+    }
 
-        Ok(())
+    /// Execute the request, returning a future resolving to a [`Response`].
+    ///
+    /// [`Response`]: crate::response::Response
+    pub fn exec(self) -> ResponseFuture<Channel> {
+        match self.request() {
+            Ok(request) => self.http.request(request),
+            Err(source) => ResponseFuture::error(source),
+        }
     }
 }
 
-impl<'a> AuditLogReason for UpdateChannel<'a> {
-    fn reason(mut self, reason: impl Into<String>) -> Result<Self, AuditLogReasonError> {
-        self.reason
-            .replace(AuditLogReasonError::validate(reason.into())?);
+impl<'a> AuditLogReason<'a> for UpdateChannel<'a> {
+    fn reason(mut self, reason: &'a str) -> Result<Self, AuditLogReasonError> {
+        self.reason.replace(AuditLogReasonError::validate(reason)?);
 
         Ok(self)
     }
 }
-
-poll_req!(UpdateChannel<'_>, Channel);
