@@ -4,6 +4,7 @@ mod create_webhook;
 mod delete_webhook;
 mod delete_webhook_message;
 mod execute_webhook;
+mod execute_webhook_and_wait;
 mod get_channel_webhooks;
 mod get_webhook;
 mod get_webhook_message;
@@ -13,8 +14,8 @@ mod update_webhook_with_token;
 pub use self::{
     create_webhook::CreateWebhook, delete_webhook::DeleteWebhook,
     delete_webhook_message::DeleteWebhookMessage, execute_webhook::ExecuteWebhook,
-    get_channel_webhooks::GetChannelWebhooks, get_webhook::GetWebhook,
-    get_webhook_message::GetWebhookMessage, update_webhook::UpdateWebhook,
+    execute_webhook_and_wait::ExecuteWebhookAndWait, get_channel_webhooks::GetChannelWebhooks,
+    get_webhook::GetWebhook, get_webhook_message::GetWebhookMessage, update_webhook::UpdateWebhook,
     update_webhook_message::UpdateWebhookMessage,
     update_webhook_with_token::UpdateWebhookWithToken,
 };
