@@ -3,9 +3,8 @@
 use crate::{
     client::Client,
     error::Error as HttpError,
-    request::{
-        self, validate, AuditLogReason, AuditLogReasonError, Form, NullableField, Pending, Request,
-    },
+    request::{self, validate, AuditLogReason, AuditLogReasonError, Form, NullableField, Request},
+    response::{marker::EmptyBody, ResponseFuture},
     routing::Route,
 };
 use serde::Serialize;
@@ -53,16 +52,14 @@ impl UpdateWebhookMessageError {
 impl Display for UpdateWebhookMessageError {
     fn fmt(&self, f: &mut Formatter<'_>) -> FmtResult {
         match &self.kind {
-            UpdateWebhookMessageErrorType::ContentInvalid { .. } => {
+            UpdateWebhookMessageErrorType::ContentInvalid => {
                 f.write_str("message content is invalid")
             }
             UpdateWebhookMessageErrorType::EmbedTooLarge { .. } => {
                 f.write_str("length of one of the embeds is too large")
             }
-            UpdateWebhookMessageErrorType::TooManyEmbeds { embeds } => {
-                Display::fmt(&embeds.len(), f)?;
-
-                f.write_str(" embeds were provided, but only 10 may be provided")
+            UpdateWebhookMessageErrorType::TooManyEmbeds => {
+                f.write_str("only 10 embeds may be provided")
             }
         }
     }
@@ -81,42 +78,33 @@ impl Error for UpdateWebhookMessageError {
 #[non_exhaustive]
 pub enum UpdateWebhookMessageErrorType {
     /// Content is over 2000 UTF-16 characters.
-    ContentInvalid {
-        /// Provided content.
-        content: String,
-    },
+    ContentInvalid,
     /// Length of one of the embeds is over 6000 characters.
     EmbedTooLarge {
-        /// Provided embeds.
-        embeds: Vec<Embed>,
         /// Index of the embed that was too large.
         ///
-        /// This can be used to index into [`embeds`] to retrieve the bad embed.
-        ///
-        /// [`embeds`]: Self::EmbedTooLarge.embeds
+        /// This can be used to index into the provided embeds to retrieve the
+        /// invalid embed.
         index: usize,
     },
     /// Too many embeds were provided.
     ///
     /// A webhook can have up to 10 embeds.
-    TooManyEmbeds {
-        /// Provided embeds.
-        embeds: Vec<Embed>,
-    },
+    TooManyEmbeds,
 }
 
-#[derive(Default, Serialize)]
-struct UpdateWebhookMessageFields {
+#[derive(Serialize)]
+struct UpdateWebhookMessageFields<'a> {
     #[serde(skip_serializing_if = "Option::is_none")]
     allowed_mentions: Option<AllowedMentions>,
-    #[serde(skip_serializing_if = "Vec::is_empty")]
-    attachments: Vec<Attachment>,
+    #[serde(skip_serializing_if = "request::slice_is_empty")]
+    attachments: &'a [Attachment],
     #[serde(skip_serializing_if = "Option::is_none")]
-    content: Option<NullableField<String>>,
+    content: Option<NullableField<&'a str>>,
     #[serde(skip_serializing_if = "Option::is_none")]
-    embeds: Option<NullableField<Vec<Embed>>>,
+    embeds: Option<NullableField<&'a [Embed]>>,
     #[serde(skip_serializing_if = "Option::is_none")]
-    payload_json: Option<Vec<u8>>,
+    payload_json: Option<&'a [u8]>,
 }
 
 /// Update a message created by a webhook.
@@ -140,25 +128,25 @@ struct UpdateWebhookMessageFields {
 ///
 /// # #[tokio::main]
 /// # async fn main() -> Result<(), Box<dyn std::error::Error>> {
-/// # let client = Client::new("token");
+/// # let client = Client::new("token".to_owned());
 /// client.update_webhook_message(WebhookId(1), "token here", MessageId(2))
 ///     // By creating a default set of allowed mentions, no entity can be
 ///     // mentioned.
 ///     .allowed_mentions(AllowedMentions::default())
-///     .content(Some("test <@3>".to_owned()))?
+///     .content(Some("test <@3>"))?
+///     .exec()
 ///     .await?;
 /// # Ok(()) }
 /// ```
 ///
 /// [`DeleteWebhookMessage`]: super::DeleteWebhookMessage
 pub struct UpdateWebhookMessage<'a> {
-    fields: UpdateWebhookMessageFields,
-    files: Vec<(String, Vec<u8>)>,
-    fut: Option<Pending<'a, ()>>,
+    fields: UpdateWebhookMessageFields<'a>,
+    files: &'a [(&'a str, &'a [u8])],
     http: &'a Client,
     message_id: MessageId,
-    reason: Option<String>,
-    token: String,
+    reason: Option<&'a str>,
+    token: &'a str,
     webhook_id: WebhookId,
 }
 
@@ -166,23 +154,25 @@ impl<'a> UpdateWebhookMessage<'a> {
     /// Maximum number of embeds that a webhook's message may have.
     pub const EMBED_COUNT_LIMIT: usize = 10;
 
-    pub(crate) fn new(
+    pub(crate) const fn new(
         http: &'a Client,
         webhook_id: WebhookId,
-        token: impl Into<String>,
+        token: &'a str,
         message_id: MessageId,
     ) -> Self {
         Self {
             fields: UpdateWebhookMessageFields {
-                allowed_mentions: http.default_allowed_mentions(),
-                ..UpdateWebhookMessageFields::default()
+                allowed_mentions: None,
+                attachments: &[],
+                content: None,
+                embeds: None,
+                payload_json: None,
             },
-            files: Vec::new(),
-            fut: None,
+            files: &[],
             http,
             message_id,
             reason: None,
-            token: token.into(),
+            token,
             webhook_id,
         }
     }
@@ -194,25 +184,12 @@ impl<'a> UpdateWebhookMessage<'a> {
         self
     }
 
-    /// Specify an attachment already present in the target message to keep.
-    ///
-    /// If called, all unspecified attachments will be removed from the message.
-    /// If not called, all attachments will be kept.
-    #[deprecated(since = "0.5.5", note = "will be removed in favor of `attachments`")]
-    pub fn attachment(mut self, attachment: Attachment) -> Self {
-        self.fields.attachments.push(attachment);
-
-        self
-    }
-
     /// Specify multiple attachments already present in the target message to keep.
     ///
     /// If called, all unspecified attachments will be removed from the message.
     /// If not called, all attachments will be kept.
-    pub fn attachments(mut self, attachments: impl IntoIterator<Item = Attachment>) -> Self {
-        self.fields
-            .attachments
-            .extend(attachments.into_iter().collect::<Vec<Attachment>>());
+    pub const fn attachments(mut self, attachments: &'a [Attachment]) -> Self {
+        self.fields.attachments = attachments;
 
         self
     }
@@ -230,21 +207,17 @@ impl<'a> UpdateWebhookMessage<'a> {
     ///
     /// Returns an [`UpdateWebhookMessageErrorType::ContentInvalid`] error type if
     /// the content length is too long.
-    pub fn content(mut self, content: Option<String>) -> Result<Self, UpdateWebhookMessageError> {
-        if let Some(content_ref) = content.as_ref() {
+    pub fn content(mut self, content: Option<&'a str>) -> Result<Self, UpdateWebhookMessageError> {
+        if let Some(content_ref) = content {
             if !validate::content_limit(content_ref) {
                 return Err(UpdateWebhookMessageError {
-                    kind: UpdateWebhookMessageErrorType::ContentInvalid {
-                        content: content.expect("content is known to be some"),
-                    },
+                    kind: UpdateWebhookMessageErrorType::ContentInvalid,
                     source: None,
                 });
             }
         }
 
-        self.fields
-            .content
-            .replace(NullableField::from_option(content));
+        self.fields.content = Some(NullableField(content));
 
         Ok(self)
     }
@@ -272,7 +245,7 @@ impl<'a> UpdateWebhookMessage<'a> {
     /// use twilight_model::id::{MessageId, WebhookId};
     ///
     /// # #[tokio::main] async fn main() -> Result<(), Box<dyn std::error::Error>> {
-    /// # let client = Client::new("token");
+    /// # let client = Client::new("token".to_owned());
     /// let embed = EmbedBuilder::new()
     ///     .description("Powerful, flexible, and scalable ecosystem of Rust libraries for the Discord API.")
     ///     .title("Twilight")
@@ -280,7 +253,8 @@ impl<'a> UpdateWebhookMessage<'a> {
     ///     .build()?;
     ///
     /// client.update_webhook_message(WebhookId(1), "token", MessageId(2))
-    ///     .embeds(Some(vec![embed]))?
+    ///     .embeds(Some(&[embed]))?
+    ///     .exec()
     ///     .await?;
     /// # Ok(()) }
     /// ```
@@ -295,57 +269,38 @@ impl<'a> UpdateWebhookMessage<'a> {
     ///
     /// [the discord docs]: https://discord.com/developers/docs/resources/channel#embed-limits
     /// [`EMBED_COUNT_LIMIT`]: Self::EMBED_COUNT_LIMIT
-    pub fn embeds(mut self, embeds: Option<Vec<Embed>>) -> Result<Self, UpdateWebhookMessageError> {
+    pub fn embeds(
+        mut self,
+        embeds: Option<&'a [Embed]>,
+    ) -> Result<Self, UpdateWebhookMessageError> {
         if let Some(embeds_present) = embeds.as_deref() {
             if embeds_present.len() > Self::EMBED_COUNT_LIMIT {
                 return Err(UpdateWebhookMessageError {
-                    kind: UpdateWebhookMessageErrorType::TooManyEmbeds {
-                        embeds: embeds.expect("embeds are known to be present"),
-                    },
+                    kind: UpdateWebhookMessageErrorType::TooManyEmbeds,
                     source: None,
                 });
             }
 
             for (idx, embed) in embeds_present.iter().enumerate() {
-                if let Err(source) = validate::embed(&embed) {
+                if let Err(source) = validate::embed(embed) {
                     return Err(UpdateWebhookMessageError {
-                        kind: UpdateWebhookMessageErrorType::EmbedTooLarge {
-                            embeds: embeds.expect("embeds are known to be present"),
-                            index: idx,
-                        },
+                        kind: UpdateWebhookMessageErrorType::EmbedTooLarge { index: idx },
                         source: Some(Box::new(source)),
                     });
                 }
             }
         }
 
-        self.fields
-            .embeds
-            .replace(NullableField::from_option(embeds));
+        self.fields.embeds = Some(NullableField(embeds));
 
         Ok(self)
     }
 
-    /// Attach a file to the webhook.
-    ///
-    /// This method is repeatable.
-    #[deprecated(since = "0.5.5", note = "will be removed in favor of `files`")]
-    pub fn file(mut self, name: impl Into<String>, file: impl Into<Vec<u8>>) -> Self {
-        self.files.push((name.into(), file.into()));
-
-        self
-    }
-
     /// Attach multiple files to the webhook.
-    pub fn files<N: Into<String>, F: Into<Vec<u8>>>(
-        mut self,
-        files: impl IntoIterator<Item = (N, F)>,
-    ) -> Self {
-        self.files.extend(
-            files
-                .into_iter()
-                .map(|(name, file)| (name.into(), file.into())),
-        );
+    ///
+    /// Calling this method will clear any previous calls.
+    pub const fn files(mut self, files: &'a [(&'a str, &'a [u8])]) -> Self {
+        self.files = files;
 
         self
     }
@@ -353,22 +308,24 @@ impl<'a> UpdateWebhookMessage<'a> {
     /// JSON encoded body of any additional request fields.
     ///
     /// If this method is called, all other fields are ignored, except for
-    /// [`file`]. See [Discord Docs/Create Message] and
+    /// [`files`]. See [Discord Docs/Create Message] and
     /// [`ExecuteWebhook::payload_json`].
     ///
-    /// [`file`]: Self::file
+    /// [`files`]: Self::files
     /// [`ExecuteWebhook::payload_json`]: super::ExecuteWebhook::payload_json
     /// [Discord Docs/Create Message]: https://discord.com/developers/docs/resources/channel#create-message-params
-    pub fn payload_json(mut self, payload_json: impl Into<Vec<u8>>) -> Self {
-        self.fields.payload_json.replace(payload_json.into());
+    pub const fn payload_json(mut self, payload_json: &'a [u8]) -> Self {
+        self.fields.payload_json = Some(payload_json);
 
         self
     }
 
-    fn request(&mut self) -> Result<Request, HttpError> {
+    // `self` needs to be consumed and the client returned due to parameters
+    // being consumed in request construction.
+    fn request(&mut self) -> Result<Request<'a>, HttpError> {
         let mut request = Request::builder(Route::UpdateWebhookMessage {
             message_id: self.message_id.0,
-            token: self.token.clone(),
+            token: self.token,
             webhook_id: self.webhook_id.0,
         })
         .use_authorization_token(false);
@@ -376,19 +333,27 @@ impl<'a> UpdateWebhookMessage<'a> {
         if !self.files.is_empty() || self.fields.payload_json.is_some() {
             let mut form = Form::new();
 
-            for (index, (name, file)) in self.files.drain(..).enumerate() {
-                form.file(format!("{}", index).as_bytes(), name.as_bytes(), &file);
+            for (index, (name, file)) in self.files.iter().enumerate() {
+                form.file(format!("{}", index).as_bytes(), name.as_bytes(), file);
             }
 
             if let Some(payload_json) = &self.fields.payload_json {
-                form.payload_json(&payload_json);
+                form.payload_json(payload_json);
             } else {
+                if self.fields.allowed_mentions.is_none() {
+                    self.fields.allowed_mentions = self.http.default_allowed_mentions();
+                }
+
                 let body = crate::json::to_vec(&self.fields).map_err(HttpError::json)?;
                 form.payload_json(&body);
             }
 
             request = request.form(form);
         } else {
+            if self.fields.allowed_mentions.is_none() {
+                self.fields.allowed_mentions = self.http.default_allowed_mentions();
+            }
+
             request = request.json(&self.fields)?;
         }
 
@@ -399,25 +364,25 @@ impl<'a> UpdateWebhookMessage<'a> {
         Ok(request.build())
     }
 
-    fn start(&mut self) -> Result<(), HttpError> {
-        let request = self.request()?;
-        self.fut.replace(Box::pin(self.http.verify(request)));
-
-        Ok(())
+    /// Execute the request, returning a future resolving to a [`Response`].
+    ///
+    /// [`Response`]: crate::response::Response
+    pub fn exec(mut self) -> ResponseFuture<EmptyBody> {
+        match self.request() {
+            Ok(request) => self.http.request(request),
+            Err(source) => ResponseFuture::error(source),
+        }
     }
 }
 
-impl<'a> AuditLogReason for UpdateWebhookMessage<'a> {
-    fn reason(mut self, reason: impl Into<String>) -> Result<Self, AuditLogReasonError> {
-        self.reason
-            .replace(AuditLogReasonError::validate(reason.into())?);
+impl<'a> AuditLogReason<'a> for UpdateWebhookMessage<'a> {
+    fn reason(mut self, reason: &'a str) -> Result<Self, AuditLogReasonError> {
+        self.reason.replace(AuditLogReasonError::validate(reason)?);
 
         Ok(self)
     }
 }
 
-poll_req!(UpdateWebhookMessage<'_>, ());
-
 #[cfg(test)]
 mod tests {
     use super::{UpdateWebhookMessage, UpdateWebhookMessageFields};
@@ -430,9 +395,9 @@ mod tests {
 
     #[test]
     fn test_request() {
-        let client = Client::new("token");
+        let client = Client::new("token".to_owned());
         let mut builder = UpdateWebhookMessage::new(&client, WebhookId(1), "token", MessageId(2))
-            .content(Some("test".to_owned()))
+            .content(Some("test"))
             .expect("'test' content couldn't be set")
             .reason("reason")
             .expect("'reason' is not a valid reason");
@@ -440,14 +405,14 @@ mod tests {
 
         let body = UpdateWebhookMessageFields {
             allowed_mentions: None,
-            attachments: Vec::new(),
-            content: Some(NullableField::Value("test".to_owned())),
+            attachments: &[],
+            content: Some(NullableField(Some("test"))),
             embeds: None,
             payload_json: None,
         };
         let route = Route::UpdateWebhookMessage {
             message_id: 2,
-            token: "token".to_owned(),
+            token: "token",
             webhook_id: 1,
         };
         let expected = Request::builder(route)
@@ -456,6 +421,6 @@ mod tests {
             .build();
 
         assert_eq!(expected.body, actual.body);
-        assert_eq!(expected.path, actual.path);
+        assert_eq!(expected.route, actual.route);
     }
 }
