@@ -1,7 +1,7 @@
 use crate::{
     client::Client,
-    error::Error,
-    request::{Pending, Request},
+    request::Request,
+    response::{marker::ListBody, ResponseFuture},
     routing::Route,
 };
 use twilight_model::{guild::Ban, id::GuildId};
@@ -18,37 +18,31 @@ use twilight_model::{guild::Ban, id::GuildId};
 ///
 /// # #[tokio::main]
 /// # async fn main() -> Result<(), Box<dyn std::error::Error>> {
-/// let client = Client::new("my token");
+/// let client = Client::new("my token".to_owned());
 ///
 /// let guild_id = GuildId(1);
 ///
-/// let bans = client.bans(guild_id).await?;
+/// let bans = client.bans(guild_id).exec().await?;
 /// # Ok(()) }
 /// ```
 pub struct GetBans<'a> {
-    fut: Option<Pending<'a, Vec<Ban>>>,
     guild_id: GuildId,
     http: &'a Client,
 }
 
 impl<'a> GetBans<'a> {
-    pub(crate) fn new(http: &'a Client, guild_id: GuildId) -> Self {
-        Self {
-            fut: None,
-            guild_id,
-            http,
-        }
+    pub(crate) const fn new(http: &'a Client, guild_id: GuildId) -> Self {
+        Self { guild_id, http }
     }
 
-    fn start(&mut self) -> Result<(), Error> {
+    /// Execute the request, returning a future resolving to a [`Response`].
+    ///
+    /// [`Response`]: crate::response::Response
+    pub fn exec(self) -> ResponseFuture<ListBody<Ban>> {
         let request = Request::from_route(Route::GetBans {
             guild_id: self.guild_id.0,
         });
 
-        self.fut.replace(Box::pin(self.http.request(request)));
-
-        Ok(())
+        self.http.request(request)
     }
 }
-
-poll_req!(GetBans<'_>, Vec<Ban>);
