@@ -1,7 +1,7 @@
 use crate::{
     client::Client,
-    error::Error as HttpError,
-    request::{validate, Pending, Request},
+    request::{validate, Request},
+    response::ResponseFuture,
     routing::Route,
 };
 use serde::Serialize;
@@ -135,15 +135,6 @@ pub struct RoleFields {
     pub position: Option<i64>,
 }
 
-impl From<RoleFieldsBuilder> for RoleFields {
-    /// Convert a [`RoleFieldsBuilder`] into a [`RoleFields`].
-    ///
-    /// This is equivalent to calling [`RoleFieldsBuilder::build`].
-    fn from(builder: RoleFieldsBuilder) -> Self {
-        builder.build()
-    }
-}
-
 /// Variants of channel fields sent to Discord.
 ///
 /// Use [`GuildChannelFieldsBuilder`] to build one.
@@ -157,8 +148,7 @@ pub enum GuildChannelFields {
 }
 
 impl GuildChannelFields {
-    #[allow(clippy::missing_const_for_fn)]
-    pub fn id(self) -> ChannelId {
+    pub const fn id(&self) -> ChannelId {
         match self {
             Self::Category(c) => c.id,
             Self::Text(t) => t.id,
@@ -201,12 +191,6 @@ pub struct TextFields {
     pub topic: Option<String>,
 }
 
-impl From<TextFieldsBuilder> for TextFields {
-    fn from(builder: TextFieldsBuilder) -> TextFields {
-        builder.build()
-    }
-}
-
 /// Voice channel fields sent to Discord.
 ///
 /// Use [`VoiceFieldsBuilder`] to build one.
@@ -226,28 +210,17 @@ pub struct VoiceFields {
     pub user_limit: Option<u64>,
 }
 
-impl From<VoiceFieldsBuilder> for VoiceFields {
-    fn from(builder: VoiceFieldsBuilder) -> VoiceFields {
-        builder.build()
-    }
-}
-
 /// Create a new request to create a guild.
 ///
 /// The minimum length of the name is 2 UTF-16 characters and the maximum is 100 UTF-16 characters.
 /// This endpoint can only be used by bots in less than 10 guilds.
 pub struct CreateGuild<'a> {
     fields: CreateGuildFields,
-    fut: Option<Pending<'a, PartialGuild>>,
     http: &'a Client,
 }
 
 impl<'a> CreateGuild<'a> {
-    pub(crate) fn new(http: &'a Client, name: impl Into<String>) -> Result<Self, CreateGuildError> {
-        Self::_new(http, name.into())
-    }
-
-    fn _new(http: &'a Client, name: String) -> Result<Self, CreateGuildError> {
+    pub(crate) fn new(http: &'a Client, name: String) -> Result<Self, CreateGuildError> {
         if !validate::guild_name(&name) {
             return Err(CreateGuildError {
                 kind: CreateGuildErrorType::NameInvalid { name },
@@ -268,20 +241,19 @@ impl<'a> CreateGuild<'a> {
                 system_channel_flags: None,
                 verification_level: None,
             },
-            fut: None,
             http,
         })
     }
 
     /// Add a role to the list of roles.
-    pub fn add_role(mut self, role: impl Into<RoleFields>) -> Self {
+    pub fn add_role(mut self, role: RoleFields) -> Self {
         if self.fields.roles.is_none() {
-            let builder = RoleFieldsBuilder::new("@everyone");
+            let builder = RoleFieldsBuilder::new("@everyone".to_owned());
             self.fields.roles.replace(vec![builder.build()]);
         }
 
         if let Some(roles) = self.fields.roles.as_mut() {
-            roles.push(role.into());
+            roles.push(role);
         }
 
         self
@@ -292,15 +264,15 @@ impl<'a> CreateGuild<'a> {
     /// This must be an ID specified in [`channels`].
     ///
     /// [`channels`]: Self::channels
-    pub fn afk_channel_id(mut self, afk_channel_id: ChannelId) -> Self {
-        self.fields.afk_channel_id.replace(afk_channel_id);
+    pub const fn afk_channel_id(mut self, afk_channel_id: ChannelId) -> Self {
+        self.fields.afk_channel_id = Some(afk_channel_id);
 
         self
     }
 
     /// Set the AFK timeout, in seconds.
-    pub fn afk_timeout(mut self, afk_timeout: u64) -> Self {
-        self.fields.afk_timeout.replace(afk_timeout);
+    pub const fn afk_timeout(mut self, afk_timeout: u64) -> Self {
+        self.fields.afk_timeout = Some(afk_timeout);
 
         self
     }
@@ -320,14 +292,15 @@ impl<'a> CreateGuild<'a> {
     ///     },
     /// };
     /// # #[tokio::main] async fn main() -> Result<(), Box<dyn std::error::Error>> {
-    /// # let client = Client::new("my token");
+    /// # let client = Client::new("my token".to_owned());
     ///
-    /// let text = TextFieldsBuilder::new("text channel")?;
-    /// let voice = VoiceFieldsBuilder::new("voice channel")?;
-    /// let text2 = TextFieldsBuilder::new("other text channel")?
-    ///     .topic("posting")?;
+    /// let text = TextFieldsBuilder::new("text channel".to_owned())?.build();
+    /// let voice = VoiceFieldsBuilder::new("voice channel".to_owned())?.build();
+    /// let text2 = TextFieldsBuilder::new("other text channel".to_owned())?
+    ///     .topic("posting".to_owned())?
+    ///     .build();
     ///
-    /// let category = CategoryFieldsBuilder::new("category channel")?
+    /// let category = CategoryFieldsBuilder::new("category channel".to_owned())?
     ///     .add_text(text2)
     ///     .add_voice(voice);
     ///
@@ -336,7 +309,10 @@ impl<'a> CreateGuild<'a> {
     ///     .add_category_builder(category)
     ///     .build();
     ///
-    /// let guild = client.create_guild("guild name")?.channels(channels)?.await?;
+    /// let guild = client.create_guild("guild name".to_owned())?
+    ///     .channels(channels)?
+    ///     .exec()
+    ///     .await?;
     /// # Ok(()) }
     /// ```
     ///
@@ -362,25 +338,21 @@ impl<'a> CreateGuild<'a> {
     /// information.
     ///
     /// [the discord docs]: https://discord.com/developers/docs/resources/guild#create-guild
-    pub fn default_message_notifications(
+    pub const fn default_message_notifications(
         mut self,
         default_message_notifications: DefaultMessageNotificationLevel,
     ) -> Self {
-        self.fields
-            .default_message_notifications
-            .replace(default_message_notifications);
+        self.fields.default_message_notifications = Some(default_message_notifications);
 
         self
     }
 
     /// Set the explicit content filter level.
-    pub fn explicit_content_filter(
+    pub const fn explicit_content_filter(
         mut self,
         explicit_content_filter: ExplicitContentFilter,
     ) -> Self {
-        self.fields
-            .explicit_content_filter
-            .replace(explicit_content_filter);
+        self.fields.explicit_content_filter = Some(explicit_content_filter);
 
         self
     }
@@ -392,8 +364,8 @@ impl<'a> CreateGuild<'a> {
     /// for more information.
     ///
     /// [the discord docs]: https://discord.com/developers/docs/reference#image-data
-    pub fn icon(mut self, icon: impl Into<String>) -> Self {
-        self.fields.icon.replace(icon.into());
+    pub fn icon(mut self, icon: String) -> Self {
+        self.fields.icon.replace(icon);
 
         self
     }
@@ -405,12 +377,12 @@ impl<'a> CreateGuild<'a> {
     /// If there are roles, this replaces the first role in the position.
     ///
     /// [`roles`]: Self::roles
-    pub fn override_everyone(mut self, everyone: impl Into<RoleFields>) -> Self {
+    pub fn override_everyone(mut self, everyone: RoleFields) -> Self {
         if let Some(roles) = self.fields.roles.as_mut() {
             roles.remove(0);
-            roles.insert(0, everyone.into());
+            roles.insert(0, everyone);
         } else {
-            self.fields.roles.replace(vec![everyone.into()]);
+            self.fields.roles.replace(vec![everyone]);
         }
 
         self
@@ -421,17 +393,15 @@ impl<'a> CreateGuild<'a> {
     /// This must be an ID specified in [`channels`].
     ///
     /// [`channels`]: Self::channels
-    pub fn system_channel_id(mut self, system_channel_id: ChannelId) -> Self {
-        self.fields.system_channel_id.replace(system_channel_id);
+    pub const fn system_channel_id(mut self, system_channel_id: ChannelId) -> Self {
+        self.fields.system_channel_id = Some(system_channel_id);
 
         self
     }
 
     /// Set the guild's [`SystemChannelFlags`].
-    pub fn system_channel_flags(mut self, system_channel_flags: SystemChannelFlags) -> Self {
-        self.fields
-            .system_channel_flags
-            .replace(system_channel_flags);
+    pub const fn system_channel_flags(mut self, system_channel_flags: SystemChannelFlags) -> Self {
+        self.fields.system_channel_flags = Some(system_channel_flags);
 
         self
     }
@@ -445,10 +415,10 @@ impl<'a> CreateGuild<'a> {
     /// ```rust,no_run
     /// use twilight_http::{Client, request::guild::create_guild::RoleFieldsBuilder};
     /// # #[tokio::main] async fn main() -> Result<(), Box<dyn std::error::Error>> {
-    /// # let client = Client::new("my token");
+    /// # let client = Client::new("my token".to_owned());
     ///
-    /// let roles = vec![RoleFieldsBuilder::new("role 1").color(0x543923)?.build()];
-    /// client.create_guild("guild name")?.roles(roles)?.await?;
+    /// let roles = vec![RoleFieldsBuilder::new("role 1".to_owned()).color(0x543923)?.build()];
+    /// client.create_guild("guild name".to_owned())?.roles(roles)?.exec().await?;
     /// # Ok(()) }
     /// ```
     ///
@@ -466,7 +436,7 @@ impl<'a> CreateGuild<'a> {
         if let Some(prev_roles) = self.fields.roles.as_mut() {
             roles.insert(0, prev_roles.remove(0));
         } else {
-            let builder = RoleFieldsBuilder::new("@everyone");
+            let builder = RoleFieldsBuilder::new("@everyone".to_owned());
             roles.insert(0, builder.build());
         }
 
@@ -475,15 +445,17 @@ impl<'a> CreateGuild<'a> {
         Ok(self)
     }
 
-    fn start(&mut self) -> Result<(), HttpError> {
-        let request = Request::builder(Route::CreateGuild)
-            .json(&self.fields)?
-            .build();
+    /// Execute the request, returning a future resolving to a [`Response`].
+    ///
+    /// [`Response`]: crate::response::Response
+    pub fn exec(self) -> ResponseFuture<PartialGuild> {
+        let mut request = Request::builder(Route::CreateGuild);
 
-        self.fut.replace(Box::pin(self.http.request(request)));
+        request = match request.json(&self.fields) {
+            Ok(request) => request,
+            Err(source) => return ResponseFuture::error(source),
+        };
 
-        Ok(())
+        self.http.request(request.build())
     }
 }
-
-poll_req!(CreateGuild<'_>, PartialGuild);
