@@ -1,7 +1,7 @@
 use crate::{
     client::Client,
-    error::Error,
-    request::{Pending, Request},
+    request::Request,
+    response::{marker::ListBody, ResponseFuture},
     routing::Route,
 };
 use twilight_model::{id::GuildId, voice::VoiceRegion};
@@ -10,29 +10,23 @@ use twilight_model::{id::GuildId, voice::VoiceRegion};
 ///
 /// Can return VIP servers if the guild is VIP-enabled.
 pub struct GetGuildVoiceRegions<'a> {
-    fut: Option<Pending<'a, Vec<VoiceRegion>>>,
     guild_id: GuildId,
     http: &'a Client,
 }
 
 impl<'a> GetGuildVoiceRegions<'a> {
-    pub(crate) fn new(http: &'a Client, guild_id: GuildId) -> Self {
-        Self {
-            fut: None,
-            guild_id,
-            http,
-        }
+    pub(crate) const fn new(http: &'a Client, guild_id: GuildId) -> Self {
+        Self { guild_id, http }
     }
 
-    fn start(&mut self) -> Result<(), Error> {
+    /// Execute the request, returning a future resolving to a [`Response`].
+    ///
+    /// [`Response`]: crate::response::Response
+    pub fn exec(self) -> ResponseFuture<ListBody<VoiceRegion>> {
         let request = Request::from_route(Route::GetGuildVoiceRegions {
             guild_id: self.guild_id.0,
         });
 
-        self.fut.replace(Box::pin(self.http.request(request)));
-
-        Ok(())
+        self.http.request(request)
     }
 }
-
-poll_req!(GetGuildVoiceRegions<'_>, Vec<VoiceRegion>);
