@@ -1,9 +1,4 @@
-use crate::{
-    client::Client,
-    error::Error,
-    request::{PendingOption, Request},
-    routing::Route,
-};
+use crate::{client::Client, request::Request, response::ResponseFuture, routing::Route};
 use twilight_model::{guild::GuildWidget, id::GuildId};
 
 /// Get the guild widget.
@@ -12,29 +7,23 @@ use twilight_model::{guild::GuildWidget, id::GuildId};
 ///
 /// [the discord docs]: https://discord.com/developers/docs/resources/guild#get-guild-widget
 pub struct GetGuildWidget<'a> {
-    fut: Option<PendingOption<'a>>,
     guild_id: GuildId,
     http: &'a Client,
 }
 
 impl<'a> GetGuildWidget<'a> {
-    pub(crate) fn new(http: &'a Client, guild_id: GuildId) -> Self {
-        Self {
-            fut: None,
-            guild_id,
-            http,
-        }
+    pub(crate) const fn new(http: &'a Client, guild_id: GuildId) -> Self {
+        Self { guild_id, http }
     }
 
-    fn start(&mut self) -> Result<(), Error> {
+    /// Execute the request, returning a future resolving to a [`Response`].
+    ///
+    /// [`Response`]: crate::response::Response
+    pub fn exec(self) -> ResponseFuture<GuildWidget> {
         let request = Request::from_route(Route::GetGuildWidget {
             guild_id: self.guild_id.0,
         });
 
-        self.fut.replace(Box::pin(self.http.request_bytes(request)));
-
-        Ok(())
+        self.http.request(request)
     }
 }
-
-poll_req!(opt, GetGuildWidget<'_>, GuildWidget);
