@@ -1,7 +1,7 @@
 use crate::{
     client::Client,
-    error::Error,
-    request::{self, AuditLogReason, AuditLogReasonError, Pending, Request},
+    request::{self, AuditLogReason, AuditLogReasonError, Request},
+    response::{marker::EmptyBody, ResponseFuture},
     routing::Route,
 };
 use twilight_model::id::{GuildId, RoleId, UserId};
@@ -18,42 +18,46 @@ use twilight_model::id::{GuildId, RoleId, UserId};
 ///
 /// # #[tokio::main]
 /// # async fn main() -> Result<(), Box<dyn std::error::Error>> {
-/// let client = Client::new("my token");
+/// let client = Client::new("my token".to_owned());
 ///
 /// let guild_id = GuildId(1);
 /// let role_id = RoleId(2);
 /// let user_id = UserId(3);
 ///
-/// client.add_guild_member_role(guild_id, user_id, role_id).reason("test")?.await?;
+/// client.add_guild_member_role(guild_id, user_id, role_id)
+///     .reason("test")?
+///     .exec()
+///     .await?;
 /// # Ok(()) }
 /// ```
 pub struct AddRoleToMember<'a> {
-    fut: Option<Pending<'a, ()>>,
     guild_id: GuildId,
     http: &'a Client,
     role_id: RoleId,
     user_id: UserId,
-    reason: Option<String>,
+    reason: Option<&'a str>,
 }
 
 impl<'a> AddRoleToMember<'a> {
-    pub(crate) fn new(
+    pub(crate) const fn new(
         http: &'a Client,
-        guild_id: impl Into<GuildId>,
-        user_id: impl Into<UserId>,
-        role_id: impl Into<RoleId>,
+        guild_id: GuildId,
+        user_id: UserId,
+        role_id: RoleId,
     ) -> Self {
         Self {
-            fut: None,
-            guild_id: guild_id.into(),
+            guild_id,
             http,
-            role_id: role_id.into(),
-            user_id: user_id.into(),
+            role_id,
+            user_id,
             reason: None,
         }
     }
 
-    fn start(&mut self) -> Result<(), Error> {
+    /// Execute the request, returning a future resolving to a [`Response`].
+    ///
+    /// [`Response`]: crate::response::Response
+    pub fn exec(self) -> ResponseFuture<EmptyBody> {
         let mut request = Request::builder(Route::AddMemberRole {
             guild_id: self.guild_id.0,
             role_id: self.role_id.0,
@@ -61,23 +65,22 @@ impl<'a> AddRoleToMember<'a> {
         });
 
         if let Some(reason) = self.reason.as_ref() {
-            request = request.headers(request::audit_header(reason)?);
+            let header = match request::audit_header(reason) {
+                Ok(header) => header,
+                Err(source) => return ResponseFuture::error(source),
+            };
+
+            request = request.headers(header);
         }
 
-        self.fut
-            .replace(Box::pin(self.http.verify(request.build())));
-
-        Ok(())
+        self.http.request(request.build())
     }
 }
 
-impl<'a> AuditLogReason for AddRoleToMember<'a> {
-    fn reason(mut self, reason: impl Into<String>) -> Result<Self, AuditLogReasonError> {
-        self.reason
-            .replace(AuditLogReasonError::validate(reason.into())?);
+impl<'a> AuditLogReason<'a> for AddRoleToMember<'a> {
+    fn reason(mut self, reason: &'a str) -> Result<Self, AuditLogReasonError> {
+        self.reason.replace(AuditLogReasonError::validate(reason)?);
 
         Ok(self)
     }
 }
-
-poll_req!(AddRoleToMember<'_>, ());
