@@ -1,7 +1,7 @@
 use crate::{
     client::Client,
-    error::Error as HttpError,
-    request::{validate, Pending, Request},
+    request::{validate, Request},
+    response::ResponseFuture,
     routing::Route,
 };
 use serde::Serialize;
@@ -11,7 +11,7 @@ use std::{
 };
 use twilight_model::{id::GuildId, template::Template};
 
-/// Error emitted when the template can not be upated as configured.
+/// Error emitted when the template can not be updated as configured.
 #[derive(Debug)]
 pub struct UpdateTemplateError {
     kind: UpdateTemplateErrorType,
@@ -63,48 +63,32 @@ impl Error for UpdateTemplateError {}
 #[non_exhaustive]
 pub enum UpdateTemplateErrorType {
     /// Name of the template is invalid.
-    NameInvalid {
-        /// Provided name.
-        name: String,
-    },
+    NameInvalid,
     /// Description of the template is invalid.
-    DescriptionTooLarge {
-        /// Provided description.
-        description: String,
-    },
+    DescriptionTooLarge,
 }
 
 #[derive(Serialize)]
-struct UpdateTemplateFields {
-    name: Option<String>,
-    description: Option<String>,
+struct UpdateTemplateFields<'a> {
+    name: Option<&'a str>,
+    description: Option<&'a str>,
 }
 
 /// Update the template's metadata, by ID and code.
 pub struct UpdateTemplate<'a> {
-    fields: UpdateTemplateFields,
-    fut: Option<Pending<'a, Template>>,
+    fields: UpdateTemplateFields<'a>,
     guild_id: GuildId,
     http: &'a Client,
-    template_code: String,
+    template_code: &'a str,
 }
 
 impl<'a> UpdateTemplate<'a> {
-    pub(crate) fn new(
-        http: &'a Client,
-        guild_id: GuildId,
-        template_code: impl Into<String>,
-    ) -> Self {
-        Self::_new(http, guild_id, template_code.into())
-    }
-
-    fn _new(http: &'a Client, guild_id: GuildId, template_code: String) -> Self {
+    pub(crate) const fn new(http: &'a Client, guild_id: GuildId, template_code: &'a str) -> Self {
         Self {
             fields: UpdateTemplateFields {
                 name: None,
                 description: None,
             },
-            fut: None,
             guild_id,
             http,
             template_code,
@@ -119,14 +103,10 @@ impl<'a> UpdateTemplate<'a> {
     ///
     /// Returns an [`UpdateTemplateErrorType::DescriptionTooLarge`] error type
     /// if the description is too large.
-    pub fn description(self, description: impl Into<String>) -> Result<Self, UpdateTemplateError> {
-        self._description(description.into())
-    }
-
-    fn _description(mut self, description: String) -> Result<Self, UpdateTemplateError> {
+    pub fn description(mut self, description: &'a str) -> Result<Self, UpdateTemplateError> {
         if !validate::template_description(&description) {
             return Err(UpdateTemplateError {
-                kind: UpdateTemplateErrorType::DescriptionTooLarge { description },
+                kind: UpdateTemplateErrorType::DescriptionTooLarge,
             });
         }
 
@@ -141,16 +121,12 @@ impl<'a> UpdateTemplate<'a> {
     ///
     /// # Errors
     ///
-    /// Returns an [`UpdateTemplateErrorType::NameInvalid`] error type if the
+    /// Returns an [`UpdateTemplateErrorType::NameInvalid`] error type when the
     /// name is invalid.
-    pub fn name(self, name: impl Into<String>) -> Result<Self, UpdateTemplateError> {
-        self._name(name.into())
-    }
-
-    fn _name(mut self, name: String) -> Result<Self, UpdateTemplateError> {
-        if !validate::template_name(&name) {
+    pub fn name(mut self, name: &'a str) -> Result<Self, UpdateTemplateError> {
+        if !validate::template_name(name) {
             return Err(UpdateTemplateError {
-                kind: UpdateTemplateErrorType::NameInvalid { name },
+                kind: UpdateTemplateErrorType::NameInvalid,
             });
         }
 
@@ -159,18 +135,20 @@ impl<'a> UpdateTemplate<'a> {
         Ok(self)
     }
 
-    fn start(&mut self) -> Result<(), HttpError> {
-        let request = Request::builder(Route::UpdateTemplate {
+    /// Execute the request, returning a future resolving to a [`Response`].
+    ///
+    /// [`Response`]: crate::response::Response
+    pub fn exec(self) -> ResponseFuture<Template> {
+        let mut request = Request::builder(Route::UpdateTemplate {
             guild_id: self.guild_id.0,
-            template_code: self.template_code.clone(),
-        })
-        .json(&self.fields)?
-        .build();
+            template_code: self.template_code,
+        });
 
-        self.fut.replace(Box::pin(self.http.request(request)));
+        request = match request.json(&self.fields) {
+            Ok(request) => request,
+            Err(source) => return ResponseFuture::error(source),
+        };
 
-        Ok(())
+        self.http.request(request.build())
     }
 }
-
-poll_req!(UpdateTemplate<'_>, Template);
